import mx.URL
from mx.URL import *

print 'Testing URL version %s ...' % mx.URL.__version__

#
# Join tests adapted from Python 1.5 urlparse.py:
#
print ' Join tests.'
a = URL('http://a/b/c/d')
tests = (
    (a+URL('http:h'),'http://a/b/c/h'),
    (a+URL('http:g'),'http://a/b/c/g'),
    (a+URL('http:'),'http://a/b/c/d'),
    (a+URL('g'),'http://a/b/c/g'),
    (a+URL('./g'),'http://a/b/c/g'),
    (a+URL('g/'),'http://a/b/c/g/'),
    (a+URL('/g'),'http://a/g'),
    (a+URL('//g'),'http://g'),
    (a+URL('?y'),'http://a/b/c/d?y'),
    (a+URL('g?y'),'http://a/b/c/g?y'),
    (a+URL('g?y/./x'),'http://a/b/c/g?y/./x'),
    (a+URL('.'),'http://a/b/c/'),
    (a+URL('./'),'http://a/b/c/'),
    (a+URL('..'),'http://a/b/'),
    (a+URL('../'),'http://a/b/'),
    (a+URL('../g'),'http://a/b/g'),
    (a+URL('../..'),'http://a/'),
    (a+URL('../../g'),'http://a/g'),
    (a+URL('../../../g'),'http://a/g'),
    (a+URL('./../g'),'http://a/b/g'),
    (a+URL('./g/.'),'http://a/b/c/g/'),
    (a+URL('/./g'),'http://a/g'),
    (a+URL('g/./h'),'http://a/b/c/g/h'),
    (a+URL('g/../h'),'http://a/b/c/h'),
    (URL('..') + URL('file:a/b/'),'file:../a/b/'),
    (URL('a/..') + URL('b/..'),'./'),
    (URL('/') + URL('a/..') + URL('b/.'),'/b/'),
    (URL('file:abc') + URL('http:abc'),'http:abc'),
    (URL('//www.test.com'),'//www.test.com'),
    (URL('http://www.test.com') + URL('a/b/c'),'http://www.test.com/a/b/c'),
    ((URL('http://www.test.com') + URL('a/b/c')).netloc,'www.test.com'),
    )

#
# More tests adapated from SF bug report #450225: "urljoin fails RFC tests"
#
# XXX The commented tests currently fail.
#
base = URL('http://a/b/c/d;p?q')
tests = tests + (
    (base + URL('ftp:h'), 'ftp:h'),
    (base + URL('g'),   'http://a/b/c/g'),
    (base + URL('./g'), 'http://a/b/c/g'),
    (base + URL('g/'),  'http://a/b/c/g/'),
    (base + URL('/g'),  'http://a/g'),
    (base + URL('//g'), 'http://g'),
#    (base + URL('?y'),  'http://a/b/c/?y'),
#    Why not http://a/b/c/d;phttp://a/b/c/d;p?y ?
    (base + URL('g?y'), 'http://a/b/c/g?y'),
    (base + URL('#s'),  'http://a/b/c/d;p?q#s'),
    (base + URL('g#s'), 'http://a/b/c/g#s'),
    (base + URL('g?y#s'), 'http://a/b/c/g?y#s'),
#    (base + URL(';x'), 'http://a/b/c/;x'),
#    Why not http://a/b/c/d;phttp://a/b/c/d;x ?
    (base + URL('g;x'),  'http://a/b/c/g;x'),
    (base + URL('g;x?y#s'), 'http://a/b/c/g;x?y#s'),
    (base + URL('.'),  'http://a/b/c/'),
    (base + URL('./'),  'http://a/b/c/'),
    (base + URL('..'),  'http://a/b/'),
    (base + URL('../'),  'http://a/b/'),
    (base + URL('../g'),  'http://a/b/g'),
    (base + URL('../..'),  'http://a/'),
    (base + URL('../../'),  'http://a/'),
    (base + URL('../../g'),  'http://a/g'),
    (base + URL(''), base.url),
#    (base + URL('../../../g')   ,  'http://a/../g'),
#    (base + URL('../../../../g'),  'http://a/../../g'),
#    (base + URL('/./g'),  'http://a/./g'),
#    (base + URL('/../g')        ,  'http://a/../g'),
#    Why should the additional ..'s be preserved ?
    (base + URL('g.')           ,  'http://a/b/c/g.'),
    (base + URL('.g')           ,  'http://a/b/c/.g'),
    (base + URL('g..')          , 'http://a/b/c/g..'),
    (base + URL('..g')          , 'http://a/b/c/..g'),
    (base + URL('./../g')       ,  'http://a/b/g'),
    (base + URL('./g/.')        ,  'http://a/b/c/g/'),
    (base + URL('g/./h')        ,  'http://a/b/c/g/h'),
    (base + URL('g/../h')       ,  'http://a/b/c/h'),
#    (base + URL('g;x=1/./y')    ,  'http://a/b/c/g;x=1/y'),
#    (base + URL('g;x=1/../y')   ,  'http://a/b/c/y'),
#    Parameters in other locations that the last path part are
#    currently not supported.
    (base + URL('g?y/./x')      ,  'http://a/b/c/g?y/./x'),
    (base + URL('g?y/../x')     ,  'http://a/b/c/g?y/../x'),
    (base + URL('g#s/./x')      ,  'http://a/b/c/g#s/./x'),
    (base + URL('g#s/../x')     ,  'http://a/b/c/g#s/../x'),
)

works = 1
for i in range(len(tests)):
    test,result = tests[i]
    try:
        assert str(test) == result
    except:
        print 'Test',i,'failed:',test,'!=',result
        works = 0
assert works

### Component tests

print ' Component tests.'
url = URL('ftp://mal:passwd@myhost:123/myfile.tgz')
assert url.scheme == 'ftp'
assert url.user == 'mal'
assert url.passwd == 'passwd'
assert url.host == 'myhost'
assert url.port == 123
assert url.path == '/myfile.tgz'
assert url.ext == 'tgz'
assert url.file == 'myfile.tgz'

url = url + './abc/abc.html'
assert url.path == '/abc/abc.html'
assert url.ext == 'html'
assert url.file == 'abc.html'

print 'Works.'
print

### Benchmarking

import time
import urlparse
basestr = 'http://a/b/c/d'
relstr = '../../g'
print 'Benchmark: basestr=%s relstr=%s' % (repr(basestr),repr(relstr)) 

loops = range(10000)

# Calibration
t = time.clock()
for i in loops:
    urljoin
    basestr
    relstr
calibration = time.clock() - t

urljoin = urlparse.urljoin
t = time.clock()
for i in loops:
    urljoin(basestr,relstr)
print ' urlparse.urljoin(basestr,relstr):',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'

loops = range(50000)

urljoin = mx.URL.urljoin
t = time.clock()
for i in loops:
    urljoin(basestr,relstr)
print ' URL.urljoin(basestr,relstr):',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'

t = time.clock()
baseurl = URL(basestr)
for i in loops:
    baseurl + URL(relstr)
print ' baseurl+URL(relstr):',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'

t = time.clock()
baseurl = URL(basestr)
for i in loops:
    baseurl + RawURL(relstr)
print ' baseurl+RawURL(relstr):',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'

t = time.clock()
baseurl = URL(basestr)
for i in loops:
    baseurl + relstr
print ' baseurl+relstr:',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'

t = time.clock()
for i in loops:
    basestr + '/' + relstr
print ' basestr + "/" + relstr:',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'

urlstr = basestr + relstr
t = time.clock()
for i in loops:
    URL(urlstr)
print ' URL(basestr+relstr):',(time.clock() - t - calibration)/len(loops)*1000.0,'msec.'




""" CSV (Comma Separated Values) table format reader/writer.

    The two classes read and write CSV data. It's a fairly simple
    format for data exchange with nearly all spreadsheets, databases,
    organizers, etc.

    The reader class is built to be easy on the data passed in: small
    errors like missing commas don't cause an exception (but do set a
    variable to indicate this). Data is converted from CSV text into a
    an internal format which can then be extracted into different
    forms of table representation.

    The writer writes standard CSV files and knows about quoting
    rules, separator handling etc. so that interfacing to spreadsheets
    and databases should pose no problem.

    Both classes can be subclassed to enhance/modify their behaviour.

    Copyright (c) 2000-2005, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
__version__ = '1.0'
_debug = 0

import re,sys,types,exceptions
from mx import TextTools, Tools

### Errors

class Error(exceptions.StandardError):
    pass

### Reader base class

class Reader:

    # Did decoding have errors ?
    errors = 0

    # If there were errors, then this list contains the line number of
    # the lines with errors
    errorlines = None
    
    # List of lines, each being a list of strings
    lines = None

    # Width of the received data (max. number of entries per line)
    width = 0
    
    # List of column names found in the data's first row; possibly
    # filtered through .filter_header()
    columns = None

    # String of separator characters which are used to separate items
    # on an input line
    separators = ',;\t'

    # My version, which runs in exponential time for '"abcdefgh...'
    # parseitem = re.compile(r'"([^"]+|"")*"|[^",;\015\012]*')

    # Thanks to Andrew Kuchling for helping me with this "simple" RE
    parseitem = re.compile(r'"([^"]|"")*"|[^"' + separators + r'\015\012]*')

    def __init__(self, separators=None):

        if separators is not None:
            self.parseitem = re.compile(r'"([^"]|"")*"|[^"'
                                        + separators + r'\015\012]*')

    def flush(self):

        """ Empty the object and reset errors
        """
        self.lines = None
        self.width = 0
        self.errors = 0
        self.columns = None

    def load(self, file, header=1, columns=None,

             StringType=types.StringType):

        """ Read a file.

            If header is true (default), the first line of input is
            interpreted as list of column names.

            If columns is given as list of strings, these columns
            names are used.

            If both header and columns are used, columns overrides the
            columns set by reading the header line. This is useful to
            override header information from the input data.

        """
        if type(file) == StringType:
            file = open(file,'rb')
        text = file.read()
        file.close()
        self.lines = self._decode(text)
        if header:
            self.process_header()
        if columns is not None:
            self.set_columns(columns)

    def loads(self, text, header=1, columns=None):

        """ Read the given text

            If header is true (default), the first line of input is
            interpreted as list of column names.

            If columns is given as list of strings, these columns
            names are used.

            If both header and columns are used, columns overrides the
            columns set by reading the header line. This is useful to
            override header information from the input data.

        """
        self.lines = self._decode(text)
        if header:
            self.process_header()
        if columns is not None:
            self.set_columns(columns)

    def list(self):

        """ Return the current data as list of lists, each having
            self.width string entries.

            Missing entries are set to None.

        """
        width = self.width
        lines = self.lines
        table = [None] * len(lines)
        for i, row in Tools.irange(lines):
            row = row[:]
            if len(row) < width:
                row[len(row):] = [None]*(width-len(row))
            table[i] = row
        return table

    def dictionary(self):

        """ Return the current data as dictionary of lists of strings,
            with one entry for each column.

            .columns must have been set using .set_columns() or by
            processing a given CSV header.

        """
        table = {}
        lines = self.lines
        keys = self.columns
        if keys is None:
            raise Error,'no columns set'
        rows = len(lines)
        for k in keys:
            table[k] = [None] * rows
        for i, key in Tools.irange(keys):
            column = table[key]
            for j, row in Tools.irange(lines):
                if len(row) > i:
                    column[j] = row[i]
        return table

    def objects(self,constructor):

        """ Builds a list of objects by calling the given constructor
            with keywords defined by mapping column names to values for
            each input line.

            .columns must have been set using .set_columns() or by
            processing a given CSV header.

        """
        lines = self.lines
        keys = self.columns
        if keys is None:
            raise Error,'no columns set'
        objs = [None] * len(lines)
        for i,line in Tools.irange(lines):
            kws = dict(Tools.tuples(keys, line))
            objs[i] = apply(constructor,(),kws)
        return objs

    def process_header(self):

        """ Process the header data.

            This also sets the .columns attribute. The header is
            removed from the data in .lines after having been
            processed.

            The header is passed through .filter_header() before
            interpreting it as list of column names.

        """
        lines = self.lines
        if len(lines) < 1:
            raise Error,'missing header data'
        self.columns = self.filter_header(lines[0])
        del lines[0]

    def set_columns(self, columns):

        """ Sets the column names to use.

            This overrides any column names possibly given in the read
            data.

        """
        self.columns = columns

    def filter_header(self, header,

                      lower=TextTools.lower):

        """ Filter the given header line.

            The base class converts the column names to all lowercase
            and removes any whitespace included in the header.

            This method is only called in case the header was read
            from the data provided to the object.

        """
        l = [''] * len(header)
        for i,column in Tools.irange(header):
            l[i] = ''.join(lower(column).split())
        return l

    def description(self, header=1):

        """ Return a list of tuples (column name, max length) found in the
            data. 

            If header is true (default), the column names themselves
            are included in the calculation.

        """
        lines = self.lines
        columns = self.columns
        width = len(columns)
        if header:
            lengths = []
            for column in columns:
                lengths.append(len(column))
        else:
            lengths = [0] * width
        for row in self.lines:
            for i,o in Tools.irange(row[:width]):
                if len(o) > lengths[i]:
                    lengths[i] = len(o)
        return map(None,columns,lengths)

    def _decode(self,text):

        """ Decode the CSV data in text.

            Internal method. Do not use directly.
        
        """
        lines = []
        x = 0
        length = len(text)
        width = 0
        errorlines = []
        match = self.parseitem.match
        separators = self.separators
        while x < length:
            l = []
            while x <= length:
                # Find next token
                m = match(text,x)
                if not m:
                    # End of line
                    break
                y = m.regs[0][1]
                l.append(text[x:y])
                if _debug:
                    print x,repr(l[-1])
                x = y + 1
                if x > length:
                    break
                # Check validity
                if text[y:x] not in separators:
                    if text[y:y+2] == '\015\012':
                        # Handle CRLF
                        x = y + 2
                    elif text[y:x] not in '\015\012':
                        # Syntax error: missing ',' or ';'
                        # Action: skip to end of line
                        y = text.find('\012', x)
                        if y < 0:
                            y = text.find('\015', x)
                        if y < 0:
                            # Skip to end of text
                            x = length
                        else:
                            x = y + 1
                        if _debug:
                            print 'errors in',l,x
                        errorlines.append(len(lines))
                    # else: found single CR or LF
                    break
            if len(l) > width:
                width = len(l)
            if _debug:
                print 'adding',l,x,repr(text[x:x+5])
            lines.append(l)
        self.width = width
        if errorlines:
            self.errors = 1
            self.errorlines = errorlines
        return map(self._unquote,lines)

    def _unquote(self,line):

        """ Unquote a CSV style quoted line of text.

            Internal method. Do not use directly.
        
        """
        for i,text in Tools.irange(line):
            if text[:1] == '"' and text[-1:] == '"':
                text = text[1:-1]
            line[i] = text.replace('""','"')
        return line

    def __len__(self):

        return len(self.lines)

    def __str__(self):

        lines = self.list()
        desc = self.description()
        width = 0
        output = []
        write = output.append
        for col in desc:
            write('%-*s|' % (col[1],col[0]))
        write('\n')
        for col in desc:
            write('=' * col[1] + '+')
        write('\n')
        for line in lines:
            for i,item in Tools.irange(line):
                write('%-*s|' % (desc[i][1],item))
            write('\n')
        return ''.join(output)

### Writer base class

class Writer:

    # Column names
    columns = None

    # CSV text
    text = ''

    # Separator to use for separating fields of a row (default: comma)
    separator = ','

    # End-of-line marker to use (default: CRLF)
    lineend = '\015\012'

    def __init__(self, separator=None, lineend=None):

        if separator is not None:
            self.separator = separator
        if lineend is not None:
            self.lineend = lineend

    def flush(self):

        """ Flush the data currently stored in the writer.
        """
        self.text = ''
        self.columns = None

    def set_columns(self, columns, header=1):

        """ Sets the output columns.

            If header is true, a column name line is added to the
            output.

            Columns can only be set once per session and must be set
            prior to adding any data. columns has to be a list of
            column names.

            It is assured that no more than len(columns) items are
            written for each row. All rows are filled up with ""
            entries to have an equal number of items.

        """
        if columns == self.columns and not header:
            # Nothing to do
            return
        elif self.columns:
            raise Error,'cannot write columns more than once per session'
        self.columns = columns
        if header:
            if self.text:
                raise Error,'cannot add header to already written data'
            headerline = self._quote(columns)
            self.text = self.separator.join(headerline) + self.lineend

    def feed_list(self,table):

        """ Feeds a table (list of rows) which is converted
            to CSV. 

            No more than len(columns) items are written for each
            row. All rows are filled up with "" entries to have an
            equal number of items. None entries are converted to empty
            strings, all other objects are stringified.

        """
        columns = self.columns
        if columns:
            rowlen = len(columns)
        else:
            # Calculate the max. number of columns in the table
            rowlen = max(map(len,table))

        # Prepare an empty table
        t = [None] * len(table)
        _quote = self._quote

        # Fill in data
        for i,row in Tools.irange(table):
            row = _quote(row[:rowlen])
            if len(row) < rowlen:
                row[len(row):] = ['""'] * (rowlen - len(row))
            t[i] = self.separator.join(row)

        # Add final CRLF and add as CSV text
        t.append('')
        self.text = self.text + self.lineend.join(t)

    def feed_dict(self,table,rows=None):

        """ Feeds a table (dict of lists) which is converted
            to CSV. 

            Only the keys set as column names are used to form the CSV
            data.

            All lists in the dictionary must have equal length or at
            least rows number of entries, if rows is given. None
            entries are converted to empty strings, all other objects
            are stringified.

        """
        columns = self.columns
        if not columns:
            raise Error,'no output columns set'
        rowlen = len(columns)

        # Create an emtpy table
        if not rows:
            rows = 0
            for column in columns:
                nrows = len(table[column])
                if nrows > rows:
                    rows = nrows
        rowindices = Tools.trange(rows)
        t = [None] * rows
        for i in rowindices:
            t[i] = [None] * rowlen
            
        # Fill the table
        for j,k in Tools.irange(columns):
            for i in rowindices:
                t[i][j] = table[k][i]
                
        # Quote and join lines
        t = [self.separator.join(self._quote(x)) for x in t]

        # Add final CRLF and store CSV text
        t.append('')
        self.text = self.text + self.lineend.join(t)

    def feed_objects(self,objects,

                     getattr=getattr):

        """ Feeds a sequence of objects which is converted to CSV. 

            For each object the set column names are interpreted as
            object attributes and used as basis for the CSV data.

            None values are converted to empty strings, all other
            attributes are added stringified.

        """
        columns = self.columns
        if not columns:
            raise Error,'no output columns set'
        rowlen = len(columns)

        # Create an emtpy table
        rows = len(objects)
        rowindices = Tools.trange(rows)
        t = [None] * rows
        for i in rowindices:
            t[i] = [None] * rowlen
            
        # Fill the table
        icols = Tools.irange(columns)
        for i in rowindices:
            obj = objects[i]
            for j,name in icols:
                t[i][j] = str(getattr(obj, name))
                
        # Quote and join lines
        t = [self.separator.join(self._quote(x)) for x in t]

        # Add final CRLF and store CSV text
        t.append('')
        self.text = self.text + self.lineend.join(t)

    def dumps(self):

        """ Returns the data as CSV text
        """
        return self.text

    def dump(self, file,

             StringType=types.StringType):

        """ Write the converted CSV data to a file
        """
        if type(file) == StringType:
            file = open(file,'wb')
            file.write(self.text)
            file.close()
        else:
            file.write(self.text)

    def _quote(self, line,

               str=str):

        """ CSV style quote the given line of text.
        """
        nline = ['""'] * len(line)
        for i,item in Tools.irange(line):
            if item is not None:
                text = str(item)
            else:
                text = ''
            nline[i] = '"%s"' % text.replace('"','""')
        return nline

def _test():
    import sys
    global _debug

    # Turn on debugging output
    #_debug = 1

    s = """"key1","key2",key3,key4
"abc",0,,0,"def""ghi
Eine neue Zeile
0","hallo",
"line",with"errors,
"text with,embedded,commas",2,3,4
newline,new,luck,;123,""\015
\"""Quote""\",1,2,3,"""#"

    r = Reader()
    r.loads(s)
    lt = r.list()
    print lt
    print r.dictionary()
    r.set_columns(('Name','Vorname'))
    print r.dictionary()
    print r.description()
    print '-' * 72


    w = Writer()
    w.feed_list(r.list())
    w.feed_list(r.list())
    s = w.dumps()
    print s

    #w.dump('test.txt')
    w.flush()
    dict = r.dictionary()
    w.set_columns(dict.keys())
    w.feed_dict(dict)
    s = w.dumps()
    print '-->',s

    r.flush()
    r.loads(s)
    lt = r.list()
    print lt
    print r.dictionary()
    r.set_columns(('Name','Vorname'))
    print r.dictionary()
    print r.description()
    print '-' * 72

    class Obj:
        pass
    data = []
    for i in range(10):
        o = Obj()
        o.id = i
        o.str = str(i)
        data.append(o)
    w.flush()
    w.set_columns(('id', 'str'))
    w.feed_objects(data)
    w.feed_objects(data)
    s = w.dumps()
    print '-->',s
    r.flush()
    r.loads(s)
    lt = r.list()
    print lt
    print r.dictionary()
    r.set_columns(('id','str'))
    print r.dictionary()
    print r.description()
    print '-' * 72


    print
    print 'Read File:',sys.argv[1]
    r.flush()
    f = open(sys.argv[1])
    r.load(f)
    print r

if __name__ == '__main__':
    _test()



from mx.DateTime import now
from time import time, clock

rounds = range(10000)

# Warm-up
for i in rounds:
    x = now()
for i in rounds:
    x = time()

c = clock()

for i in rounds:
    x = now()

print 'now(): %5.2f sec.' % (clock() - c)

c = clock()

for i in rounds:
    x = time()

print 'time(): %5.2f sec.' % (clock() - c)




#!/usr/local/bin/python -u
""" Y2000.py - The year 2000 countdown.
"""
from mx.DateTime import *
from time import sleep
import sys

while 1:
    d = Date(2000,1,1) - now()
    if d.days < 0:
        print 'Y2000... made it !'
        break
    print 'Y2000... time left: %2i days %2i hours %2i minutes %2i seconds\r'%\
          (d.day,d.hour,d.minute,d.second),
    sys.stdout.flush()
    sleep(1)



#!/usr/local/bin/python

""" HTML - tag a HTML string (Version 0.6)
    
    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""

import sys

# constants + engine
from mx.TextTools import *

# ErrorTag
error = '*syntax error'                 # error tag obj

tagname_charset = CharSet(alpha+'\-'+number)
tagattrname_charset = CharSet(alpha+'\-'+number)
tagvalue_charset = CharSet('^"\'> ')
white_charset = CharSet(' \r\n\t')

tagname_charset = CharSet(alpha+'\-'+number)
tagattrname_charset = CharSet(alpha+'\-'+number)
tagvalue_charset = CharSet('^"\'> ')
white_charset = CharSet(' \r\n\t')

tagattr = (
       # name
       ('name',AllInCharSet,tagattrname_charset),
       # with value ?
       (None,Is,'=',MatchOk),
       # skip junk
       (None,AllInCharSet,white_charset,+1),
       # unquoted value
       ('value',AllInCharSet,tagvalue_charset,+1,MatchOk),
       # double quoted value
       (None,Is,'"',+5),
         ('value',AllNotIn,'"',+1,+2),
         ('value',Skip,0),
         (None,Is,'"'),
         (None,Jump,To,MatchOk),
       # single quoted value
       (None,Is,'\''),
         ('value',AllNotIn,'\'',+1,+2),
         ('value',Skip,0),
         (None,Is,'\'')
       )

valuetable = (
    # ignore whitespace + '='
    (None,AllInCharSet,CharSet(' \r\n\t='),+1),
    # unquoted value
    ('value',AllInCharSet,tagvalue_charset,+1,MatchOk),
    # double quoted value
    (None,Is,'"',+5),
     ('value',AllNotIn,'"',+1,+2),
     ('value',Skip,0),
     (None,Is,'"'),
     (None,Jump,To,MatchOk),
    # single quoted value
    (None,Is,'\''),
     ('value',AllNotIn,'\'',+1,+2),
     ('value',Skip,0),
     (None,Is,'\'')
    )

allattrs = (
    # look for attributes
    (None,AllInCharSet,white_charset,+4),
     (None,Is,'>',+1,MatchOk),
     ('tagattr',Table,tagattr),
     (None,Jump,To,-3),
    (None,Is,'>',+1,MatchOk),
    # handle incorrect attributes
    (error,AllNotIn,'> \r\n\t'),
    (None,Jump,To,-6)
    )

# NOTE: The htmltag tag table assumes that the input text is given 
#       in upper case letters (see <XMP> handling).

htmltag = (
    (None,Is,'<'),
    # is this a closing tag ?
    ('closetag',Is,'/',+1),
    # a coment ?
    ('comment',Is,'!',+8),
     (None,Word,'--',+4),
     ('text',WordStart,'-->',+1),
     (None,Skip,3),
     (None,Jump,To,MatchOk),
     # a SGML-Tag ?
     ('other',AllNotIn,'>',+1),
      (None,Is,'>'),
     (None,Jump,To,MatchOk),
    # XMP-Tag ?
    ('tagname',Word,'XMP',+5),
     (None,Is,'>'),
     ('text',WordStart,'</XMP>'),
     (None,Skip,len('</XMP>')),
     (None,Jump,To,MatchOk),
    # get the tag name
    ('tagname',AllInCharSet,tagname_charset),
    # look for attributes
    (None,AllInCharSet,white_charset,+4),
     (None,Is,'>',+1,MatchOk),
     ('tagattr',Table,tagattr),
     (None,Jump,To,-3),
     (None,Is,'>',+1,MatchOk),
    # handle incorrect attributes
    (error,AllNotIn,'> \n\r\t'),
    (None,Jump,To,-6)
    )

htmltable = (# HTML-Tag
             ('htmltag',Table,htmltag,+1,+4),
             # not HTML, but still using this syntax: error or inside XMP-tag !
             (error,Is,'<',+3),
              (error,AllNotIn,'>',+1),
              (error,Is,'>'),
             # normal text
             ('text',AllNotIn,'<',+1),
             # end of file
             ('eof',EOF,Here,-5),
            )

if __name__ == '__main__':

    t = TextTools._timer()

    # read file
    f = open(sys.argv[1])
    text = f.read()

    try:
        count = int(sys.argv[2])
    except:
        count = 1000
    
    print 'Starting to parse the file %i times...' % count

    # parse file
    t.start()
    for i in range(count):
        utext = upper(text)
        result, taglist, nextindex = tag(utext,htmltable)
        if not result:
            print ' parsing failed; aborting'
            break
    t = t.stop()[0]

    mean = t/count
    print result, nextindex, mean*1000,'msec',nextindex/mean,'bytes/sec.'
    print
    print 'Hit return to see the tags...'
    raw_input()
    print
    print_tags(text,taglist)



""" mx.UID -- A UID datatype.

    Relies on mx.DateTime.

    Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.

"""
import mimetypes
try:
    import hashlib
except ImportError:
    import md5 as hashlib
from mx import DateTime

#
# Import the C extension module
#
from mxUID import *
from mxUID import __version__

### Helpers

def mangle(uid, key,

           md5=hashlib.md5,otp=otp):

    """ Mangle the UID string uid using the given key string.

        The output has the same length as the input UID string and
        should make it hard to forge valid UIDs without knowledge of
        the key string.

        Note that the output string is not a valid UID string in
        itself, i.e. it most likely won't verify().

    """
    # Idea: Even if somebody finds the akey which allows decoding
    # the timestamp, it should not be possible to use it to extract
    # the key or the bkey from it.
    akey = md5(key).digest()
    bkey = md5(uid[:16] + key).digest()
    # Add a little noise to please the eye (except to the counter part)
    # and apply the encoding pad
    ckey = 'ffff' + uid[:4] * 7
    return otp(otp(uid, ckey), akey + bkey * 2)

def demangle(muid, key,

             md5=hashlib.md5,otp=otp):

    """ Demangle a mangle()d UID string muid using the given key
        string.

    """
    akey = md5(key).digest()
    # First decode the counter and the noisy timestamp part
    tuid = otp(muid[:16], akey)
    # Next denoise the timestamp part to build the bkey
    ckey = 'ffff' + tuid[:4] * 7
    bkey = md5(otp(tuid, ckey) + key).digest()
    # Decode and denoise the UID
    return otp(otp(muid, akey + bkey * 2), ckey)

# Override the function provided by mxUID with one using DateTime
# instances:
timestamp_ticks = timestamp
def timestamp(uid,

              DateTimeFromTicks=DateTime.DateTimeFromTicks,
              timestamp=timestamp_ticks):

    """ Returns the timestamp encoded in the UID string uid
        as DateTime instance.
    """
    return DateTimeFromTicks(timestamp(uid))

###

#
# Experimental Python wrapper around UID strings.
#

class ID:

    """ ID class for creating unique IDs.

        The generated IDs are unique to the host, process and time
        with high probabilty. In addition their validity can be
        verified (using a CRC type algorithm).

        The algorithm uses a 40-bit timestamp in the ID which can be
        extracted using the .timestamp() method. Accuracy is one second.

    """
    # ID value
    uid = None 

    def __init__(self,target=None,code='',timestamp=None):

        """ Construct a new uid for the object target.

            target defaults to the singleton None.

            code is an optional addition to the uid that can later be
            used to verify the validity of the uid together with the
            UID's CRC value.

            timestamp may be given as DateTime instance. It defaults
            to the current local time.

        """
        if timestamp:
            self.uid = UID(target,code,timestamp)
        else:
            self.uid = UID(target,code)

    def timestamp(self,

                  DateTimeFromTicks=DateTime.DateTimeFromTicks,
                  timestamp=timestamp):

        """ Returns the timestamp encoded in the ID
            as DateTime instance.
        """
        return DateTimeFromTicks(timestamp(self.uid))

    def set_uid(self,uid,code='',

               verify=verify):

        # Check validity
        if not verify(uid,code):
            raise Error,'invalid UID'

        self.uid = uid

    def __str__(self):

        return self.uid

    def __cmp__(self,other,

                cmp=cmp):

        return cmp(self.uid,other.uid)

    def __hash__(self,

                 hash=hash):

        return hash(self.uid)

# Helper class
class _EmptyClass:
    pass

### Other constructors

def IDFromUID(s,code=''):

    """ Create an ID object from the given string UID.

        This can raise an Error in case the string does not map to a
        valid UID. code is used in the verification process if given.

    """
    id = _EmptyClass()
    id.__class__ = ID
    id.set_uid(s,code)
    return id

if __name__ == '__main__':
    uid = UID()
    print uid,' timestamp =',timestamp(uid)
    print verify(uid) * 'OK' or 'NOT OK'



""" mxURL - A URL datatype.

    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.

"""
from URL import *
from URL import __version__

#
# Make URLs pickleable
#

# Shortcut for pickle (reduces the pickle's length)
def _URL(url,

         RawURL=RawURL):

    return RawURL(url)

# Module init
class _modinit:

    ### Register the type
    import copy_reg
    def pickle_URL(url):
        return _URL,(url.string,)
    copy_reg.pickle(URLType,
                    pickle_URL,
                    _URL)

del _modinit



import hack
import mx.Tools.NewBuiltins

a = trange(10000)

def f1():

    x = (a,a,a)
    for i in trange(100):
        x = lists(x)

def f2():

    x = (a,a,a)
    for i in trange(100):
        x = tuples(x)

def f3(apply=apply,map=map,tuple=tuple):

    x = (a,a,a)
    y = None
    for i in trange(100):
        x = apply(map,(y,)+tuple(x))

print 'lists:',hack.clock('f1()')
print 'tuples:',hack.clock('f2()')
print 'map:',hack.clock('f3()')



from mx.DateTime import *
import time,sys

print 'Python',sys.version.split()[0],'on',sys.platform
print

x = time.time()
print 'ticks:',x
print 'time.localtime():',time.localtime(x)
print 'time.gmtime():',time.gmtime(x)
print 'time.mktime():',time.mktime(time.localtime(x)[:-1]+(-1,))
t = DateTimeFromTicks(x)
print 't.ticks():',t.ticks()
print 't.tuple():',t.tuple()
print 't.absdate:',t.absdate
print 't.abstime:',t.abstime
print 't.tuple() -> time.mktime():',apply(time.mktime,t.tuple())
print '          -> time.localtime():',time.localtime(apply(time.mktime,t.tuple()))
print "hasattr(Epoch,'gmticks'):",hasattr(Epoch,'gmticks')
try:
    print 'tz_offset(t):',tz_offset(t)
except:
    print 'tz_offset not working'

print

x = x - 15768000
print 'ticks:',x
print 'time.localtime():',time.localtime(x)
print 'time.gmtime():',time.gmtime(x)
print 'time.mktime():',time.mktime(time.localtime(x)[:-1]+(-1,))
t = DateTimeFromTicks(x)
print 't.ticks():',t.ticks()
print 't.tuple():',t.tuple()
print 't.absdate:',t.absdate
print 't.abstime:',t.abstime
print 't.tuple() -> time.mktime():',apply(time.mktime,t.tuple())
print '          -> time.localtime():',time.localtime(apply(time.mktime,t.tuple()))
print "hasattr(Epoch,'gmticks'):",hasattr(Epoch,'gmticks')
try:
    print 'tz_offset(t):',tz_offset(t)
except:
    print 'tz_offset not working'



#!/usr/local/bin/python -O
""" stackbench - stack implementation benchmark

    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
import time
from mx import Stack
import UserStack
from sys import argv, exit

try:
    numtests, pushes, pops = eval(argv[1]), eval(argv[2]), eval(argv[3])
    assert pushes >= pops
except:
    print 'usage: stackbench.py <ntests> <pushes> <pops>, where <pushes> >= <pops>'
    exit(1)
 
def test(reps, func):
    start_cpu  = time.clock()
    for i in xrange(reps):
        x = func()
    return time.clock() - start_cpu
 
def method1():
    x = []                                           # built-in list
    push = x.append
    for i in range(pushes): push('spam'+'i')
    for i in range(pops):   top = x[-1]; del x[-1]

if 0:
    def method1a():
        x = []                                           # built-in list
        push = x.append
        for i in range(pushes): push('spam'+'i')
        for i in range(pops):   top = x.pop()

def method2():
    x = None                                         # built-in tuples
    for i in range(pushes): x = ('spam'+'i',x)
    for i in range(pops):   (top, x) = x
 
def method3():
    s = Stack.Stack()                              # Stack
    push = s.push
    pop = s.pop
    for i in range(pushes): push('spam'+'i')
    for i in range(pops):   top = pop()

def method3a():
    s = Stack.Stack()                              # Stack
    push = s.push
    for i in range(pushes): push('spam'+'i')
    t = s.pop_many(pops)                             # pop all at once

def method3b():
    s = Stack.Stack()                              # Stack
    push = s.push
    for i in range(pushes): s << ('spam'+'i')
    for i in range(pops):   top = s >> 1

def method3c():
    s = Stack.Stack()                              # Stack
    l = [''] * pushes
    for i in range(pushes): l[i] = ('spam'+'i')
    s.push_many(l)
    s.pop_many(pops)

def method4():
    s = UserStack.UserStack()                         # UserStack
    push = s.push
    pop = s.pop
    for i in range(pushes): push('spam'+'i')
    for i in range(pops):   top = pop()

print 'list: ', test(numtests, method1)            # run func 20 tests
print 'tuples:', test(numtests, method2)
print 'Stack (with push + pop):', test(numtests, method3)
print 'Stack (with push + pop_many):', test(numtests, method3a)
print 'Stack (with << + >>):', test(numtests, method3b)
print 'Stack (with push_many + pop_many):', test(numtests, method3c)
print 'UserStack:', test(numtests, method4)
 




""" CommandLine - Get and parse command line options

    NOTE: This still is very much work in progress !!!

    Different version are likely to be incompatible.

    TODO:

    * Incorporate the changes made by (see Inbox)
    * Add number range option using srange()

"""

__copyright__ = """\
Copyright (c), 1997-2000, IKDS Marc-Andre Lemburg (mal@lemburg.com)
Copyright (c), 2000-2009, eGenix.com Software GmbH (info@egenix.com)
See the documentation for further information on copyrights,
or contact the author. All Rights Reserved.
"""

__version__ = '1.3'

import sys, getopt, glob, os, re, exceptions, traceback

# For Python < 2.2 compatibility:
try:
    True
except NameError:
    True = 1
    False = 0

### Helpers

def _getopt_flags(options):

    """ Convert the option list to a getopt flag string and long opt
        list

    """
    s = []
    l = []
    for o in options:
        if o.prefix == '-':
            # short option
            s.append(o.name)
            if o.takes_argument:
                s.append(':')
        else:
            # long option
            if o.takes_argument:
                l.append(o.name+'=')
            else:
                l.append(o.name)
    return ''.join(s),l

def invisible_input(prompt='>>> '):

    """ Get raw input from a terminal without echoing the characters to
        the terminal, e.g. for password queries.

    """
    import getpass
    entry = getpass.getpass(prompt)
    if entry is None:
        raise KeyboardInterrupt
    return entry

def fileopen(name, mode='wb', encoding=None):

    """ Open a file using mode.

        Default mode is 'wb' meaning to open the file for writing in
        binary mode. If encoding is given, I/O to and from the file is
        transparently encoded using the given encoding.

        Files opened for writing are chmod()ed to 0600.

    """
    if name == 'stdout':
        return sys.stdout
    elif name == 'stderr':
        return sys.stderr
    elif name == 'stdin':
        return sys.stdin
    else:
        if encoding is not None:
            import codecs
            f = codecs.open(name, mode, encoding)
        else:
            f = open(name, mode)
        if 'w' in mode:
            os.chmod(name, 0600)
        return f

def option_dict(options):

    """ Return a dictionary mapping option names to Option instances.
    """
    d = {}
    for option in options:
        d[option.name] = option
    return d

def parse_optionstring(optionstring):

    """ Split the optionstring into a tuple (prefix, name).

    """
    if not optionstring[:1] == '-':
        raise TypeError,'option strings must start with "-"'
    if optionstring[:2] == '--':
        prefix = '--'
        name = optionstring[2:]
    else:
        prefix = '-'
        name = optionstring[1:]
    return prefix, name

# Alias
getpasswd = invisible_input

_integerRE = re.compile('\s*(-?\d+)\s*$')
_integerRangeRE = re.compile('\s*(-?\d+)\s*-\s*(-?\d+)\s*$')

def srange(s,

           integer=_integerRE,
           integerRange=_integerRangeRE):

    """ Converts a textual representation of integer numbers and ranges
        to a Python list.

        Supported formats: 2,3,4,2-10,-1 - -3, 5 - -2

        Values are appended to the created list in the order specified
        in the string.

    """
    l = []
    append = l.append
    for entry in s.split(','):
        m = integer.match(entry)
        if m:
            append(int(m.groups()[0]))
            continue
        m = integerRange.match(entry)
        if m:
            start,end = map(int,m.groups())
            l[len(l):] = range(start,end+1)
    return l

def abspath(path,

            expandvars=os.path.expandvars,expanduser=os.path.expanduser,
            join=os.path.join,getcwd=os.getcwd):

    """ Return the corresponding absolute path for path.

        path is expanded in the usual shell ways before
        joining it with the current working directory.

    """
    try:
        path = expandvars(path)
    except AttributeError:
        pass
    try:
        path = expanduser(path)
    except AttributeError:
        pass
    return join(getcwd(), path)

### Option classes

class Option:

    """ Option base class. Takes no argument.

    """
    default = None
    helptext = ''
    prefix = '-'
    takes_argument = 0
    has_default = 0
    tab = 15

    def __init__(self, optionstring, help=None):

        self.prefix, self.name = parse_optionstring(optionstring)
        if help:
            self.help = help

    def __str__(self):

        o = self
        name = o.prefix + o.name
        if o.takes_argument:
            name = name + ' arg'
        if len(name) > self.tab:
            name = name + '\n' + ' ' * (self.tab + 1 + len(o.prefix))
        else:
            name = '%-*s ' % (self.tab, name)
        description = o.help
        if o.has_default:
            description = description + ' (%s)' % o.default
        return '%s %s' % (name, description)

    def get_optionstring(self):

        return self.prefix + self.name

class ArgumentOption(Option):

    """ Option that takes an argument.

        An optional default argument can be given.

    """
    def __init__(self,name,help=None,default=None):

        # Basemethod
        Option.__init__(self,name,help)

        if default is not None:
            self.default = default
            self.has_default = 1
        self.takes_argument = 1

class SwitchOption(Option):

    """ Options that can be on or off. Has an optional default value.

    """
    def __init__(self,name,help=None,default=None):

        # Basemethod
        Option.__init__(self,name,help)

        if default is not None:
            self.default = default
            self.has_default = 1

### Application baseclass

class Application:

    """ Command line application interface with builtin argument
        parsing.

    """
    # Options the program accepts (Option instances)
    options = []

    # Standard settings; these are appended to options in __init__
    preset_options = [SwitchOption('-v',
                                   'generate verbose output'),
                      SwitchOption('--verbose',
                                   'generate verbose output'),
                      SwitchOption('-h',
                                   'show this help text'),
                      SwitchOption('--help',
                                   'show this help text'),
                      SwitchOption('--yes',
                                   'answer yes to all confirmations'),
                      SwitchOption('--debug',
                                   'enable debugging'),
                      SwitchOption('--copyright',
                                   'show copyright'),
                      SwitchOption('--examples',
                                   'show examples of usage')]

    # The help layout looks like this:
    # [header]   - defaults to ''
    #
    # [synopsis] - formatted as '<self.name> %s' % self.synopsis
    #
    # options:
    # [options]  - formatted from self.options
    #
    # [version]  - formatted as 'Version:\n %s' % self.version, if given
    #
    # [about]    - defaults to ''
    #
    # Note: all fields that do not behave as template are formatted
    #       using the instances dictionary as substitution namespace,
    #       e.g. %(name)s will be replaced by the applications name.
    #

    # Header (default to program name)
    header = ''

    # Name (defaults to program name)
    name = ''

    # Synopsis (%(name)s is replaced by the program name)
    synopsis = '%(name)s [option] files...'

    # Version (optional)
    version = ''

    # General information printed after the possible options (optional)
    about = ''

    # Examples of usage to show when the --examples option is given (optional)
    examples = ''

    # Copyright to show
    copyright = __copyright__

    # Apply file globbing ?
    globbing = 1

    # Generate debug output ?
    debug = 0

    # Generate verbose output ?
    verbose = 0

    # Internal errors to catch
    InternalError = exceptions.Exception

    # Text to display if the user has not provided an entry value for
    # a required field
    missing_entry_text = 'missing entry'

    # Text to display if the user has entered an unsupported value
    wrong_entry_text = 'please only use the shown values'

    # Answer yes to all questions ?
    answer_yes = False

    # Instance variables:
    values = None       # Dictionary of passed options (or default values)
                        # indexed by the options name, e.g. '-h'
    files = None        # List of passed filenames
    optionlist = None   # List of passed options
    optionmap = None    # Dictionary mapping option name to Option instance

    def __init__(self,argv=None):

        # Setup application specs
        if argv is None:
            argv = sys.argv
        self.filename = os.path.split(argv[0])[1]
        if not self.name:
            self.name = os.path.split(self.filename)[1]
        else:
            self.name = self.name
        if not self.header:
            self.header = self.name
        else:
            self.header = self.header

        # Init .arguments list
        self.arguments = argv[1:]

        # Setup Option mapping
        self.optionmap = option_dict(self.options)

        # Append preset options
        for option in self.preset_options:
            if not self.optionmap.has_key(option.name):
                self.add_option(option)

        # Init .files list
        self.files = []

        # Start Application
        try:
            try:
                # Process startup
                rc = self.startup()
                if rc is not None:
                    raise SystemExit,rc

                # Parse command line
                rc = self.parse()
                if rc is not None:
                    raise SystemExit,rc

                # Start application
                rc = self.main()
                if rc is None:
                    rc = 0

            except SystemExit,rc:
                pass

            except KeyboardInterrupt:
                print
                print '* User Break'
                print
                rc = 1

            except self.InternalError:
                print
                print '* Internal Error:'
                if self.verbose or self.debug:
                    print
                    traceback.print_exc(20, sys.stdout)
                else:
                    print
                    print '  %s: %s' % sys.exc_info()[:2]
                    print
                    print '  (use --debug to see the traceback)'
                print
                rc = 1
        finally:
            new_rc = self.finished(rc)
            if new_rc is not None:
                rc = new_rc

        raise SystemExit,rc

    def add_option(self, option):

        """ Add a new Option instance to the Application dynamically.

            Note that this has to be done *before* .parse() is being
            executed.

        """
        self.options.append(option)
        self.optionmap[option.name] = option

    def startup(self):

        """ Set user defined instance variables.

            If this method returns anything other than None, the
            process is terminated with the return value as exit code.

        """
        return None

    def finished(self, rc):

        """ The program has finished with exit code rc.

            This method is always called at the end of script
            execution. An exit code of 0 means: no problems. All other
            exit codes refer to error situations.

            If this method returns anything other than None, the
            process is terminated with the return value as exit code
            (replacing the original exit code).

        """
        return None
    
    def exit(self, rc=0):

        """ Exit the program.

            rc is used as exit code and passed back to the calling
            program. It defaults to 0 which usually means: OK.

        """
        raise SystemExit, rc

    def parse(self):

        """ Parse the command line and fill in self.values and self.files.

            After having parsed the options, the remaining command line
            arguments are interpreted as files and passed to .handle_files()
            for processing.

            As final step the option handlers are called in the order
            of the options given on the command line.

        """
        # Parse arguments
        self.values = values = {}
        for o in self.options:
            if o.has_default:
                values[o.prefix+o.name] = o.default
            else:
                values[o.prefix+o.name] = 0
        flags,lflags = _getopt_flags(self.options)
        try:
            optlist,files = getopt.getopt(self.arguments,flags,lflags)
            if self.globbing:
                l = []
                for f in files:
                    gf = glob.glob(f)
                    if not gf:
                        l.append(f)
                    else:
                        l[len(l):] = gf
                files = l
            self.optionlist = optlist
            self.files = files + self.files
        except getopt.error,why:
            self.help(why)
            sys.exit(1)

        # Call file handler
        rc = self.handle_files(self.files)
        if rc is not None:
            sys.exit(rc)

        # Call option handlers
        for optionstring, value in optlist:

            # Get Option instance
            optionprefix, optionname = parse_optionstring(optionstring)
            option = self.optionmap[optionname]
            
            # Try to convert value to integer
            try:
                value = int(value)
            except ValueError:
                pass

            # Find handler and call it (or count the number of option
            # instances on the command line)
            handlername = 'handle' + optionstring.replace('-', '_')
            try:
                handler = getattr(self, handlername)
            except AttributeError:
                if value == '':
                    # count the number of occurances
                    if values.has_key(optionstring):
                        values[optionstring] = values[optionstring] + 1
                    else:
                        values[optionstring] = 1
                else:
                    values[optionstring] = value
                if self.debug:
                    print ('no option handler found for %r, '
                           'recorded value %r' %
                           (optionstring, values[optionstring]))
            else:
                if option.takes_argument:
                    if self.debug:
                        print ('calling option handler %s(%r)' %
                               (handlername, value))
                    rc = handler(value)
                else:
                    if self.debug:
                        print ('calling option handler %s()' %
                               (handlername,))
                    rc = handler()
                if rc is not None:
                    raise SystemExit, rc

        # Apply final file check (for backward compatibility)
        rc = self.check_files(self.files)
        if rc is not None:
            sys.exit(rc)

    def check_files(self, filelist):

        """ Apply some user defined checks on the files given in filelist.

            This may modify filelist in place. A typical application
            is checking that at least n files are given.

            If this method returns anything other than None, the
            process is terminated with the return value as exit code.

        """
        return None

    def help(self, note=''):

        self.print_header()
        if self.synopsis:
            print 'Synopsis:'
            # To remain backward compatible:
            try:
                synopsis = self.synopsis % self.name
            except (NameError, KeyError, TypeError):
                synopsis = self.synopsis % self.__dict__
            print ' ' + synopsis
        print
        self.print_options()
        if self.version:
            print 'Version:'
            print ' %s' % self.version
            print
        if self.about:
            print (self.about % self.__dict__).strip()
            print
        if note:
            print '-'*72
            print 'Note:',note
            print

    def notice(self, note):

        print '-'*72
        print 'Note:',note
        print '-'*72
        print

    def print_header(self):

        print '-'*72
        print self.header % self.__dict__
        print '-'*72
        print

    def print_options(self):

        options = self.options
        print 'Options and default settings:'
        if not options:
            print '  None'
            return
        long = filter(lambda x: x.prefix == '--', options)
        short = filter(lambda x: x.prefix == '-', options)
        items = short + long
        for o in options:
            print ' ',o
        print

    def ask(self, prompt, default=None, hidden=False):

        """ Ask the user for a string data entry.

            prompt is shown to the user.

            If default is given as string, the user can accept the
            default value by hitting return. The default value is
            displayed to the user after the prompt string.

            If no default is given, a value must be entered by the
            user. If he doesn't enter a value, the prompt is
            redisplayed.

            If hidden is true (default is false), the data entry is
            not displayed on the terminal. This is useful for password
            entries.

        """

        while True:
            # Show prompt and default value (if any)
            if default:
                print '%s [%s]' % (prompt, default),
            else:
                print '%s' % prompt,

            # Get entry
            if hidden:
                entry = invisible_input(' ')
            else:
                entry = raw_input()

            # Check entry
            if entry:
                break
            else:
                if default is None:
                    print '%s ... %s' % (prompt, self.missing_entry_text)
                    continue
                else:
                    entry = default
                    break
                
        return entry

    def confirm(self, prompt, default=False, answers=('no', 'yes')):

        """ Get user confirmation.

            The prompt is shown to the user and he is asked to enter
            one of the possible answers.

            Returns a boolean with the answer. True means: the user
            accepted, False: the user did not confirm.

            default defines the default value to use in case the user
            does not provide an answer and just hits enter.

            answers defines the answers to accept. It must be given as
            tuple (no_value, yes_value). Defaults to ('no', 'yes').

            Case does not matter when entering the answer.

        """
        if self.answer_yes:
            return True
        default_answer = answers[default]
        lower_cased_answers = [answer.lower() for answer in answers]
        while True:
            entry = self.ask(prompt + ' (%s/%s)' % answers, default_answer)
            entry = entry.strip().lower()
            if entry not in lower_cased_answers:
                print '%s ... %s' % (prompt, self.wrong_entry_text)
                continue
            return entry == lower_cased_answers[True]

    #
    # Example handlers:
    #
    # If a handler returns anything other than None, processing stops
    # and the return value is passed to sys.exit() as argument.
    #

    # File handler
    def handle_files(self, files):

        """ This may process the files list in place.
        """
        return None

    # Short option handler
    def handle_h(self, arg=None):

        self.help()
        return 0

    def handle_v(self, value=None):

        """ Turn on verbose output.
        """
        self.verbose = 1

    # Handlers for long options have two underscores in their name
    def handle__help(self, arg=None):

        self.help()
        return 0

    def handle__verbose(self, value=None):

        """ Turn on verbose output.
        """
        self.verbose = 1

    def handle__debug(self, arg=None):

        self.debug = 1
        # We don't want to catch internal errors:
        self.InternalError = None

    def handle__copyright(self, arg=None):

        self.print_header()
        print (self.copyright % self.__dict__).strip()
        print
        return 0

    def handle__examples(self, arg=None):

        self.print_header()
        if self.examples:
            print 'Examples:'
            print
            print (self.examples % self.__dict__).strip()
            print
        else:
            print 'No examples available.'
            print
        return 0

    def handle__yes(self, dummy=None):

        self.answer_yes = True

    def main(self):

        """ Override this method as program entry point.

            The return value is passed to sys.exit() as argument.  If
            it is None, 0 is assumed (meaning OK). Unhandled
            exceptions are reported with exit status code 1 (see
            __init__ for further details).

        """
        return None

# Alias
CommandLine = Application

def _test():

    class MyApplication(Application):
        header = 'Test Application'
        version = __version__
        options = [Option('-v','verbose')]

        def handle_v(self,arg):
            print 'VERBOSE, Yeah !'

    cmd = MyApplication()
    if not cmd.values['-h']:
        cmd.help()
    print 'files:',cmd.files
    print 'Bye...'

if __name__ == '__main__':
    _test()



# numdate.py
"""
The functions in this module are useful for data entry or bulk data
import, where dates are in an all-numeric format.  Because of the
possibility that the user might type the wrong thing in a numeric
field, or a data file might contain other data or garbage where a date
is expected, it is desirable to return None or raise an exception
when an invalid expression is encountered.

Dates can be in any of these formats: ymd, dmy, or mdy
with any single-character nondigit separator, two- or four-digit
year, and one- or two-digit month and day; or they can be in all-digits
ymd format with no separator, four-digit year, and two-digit
month and day (e.g. 20010803).

In accordance with ISO 8601, two-digit years in all-numeric dates
are interpreted as referring to the current century.

Synopsis:
    DateTime = numericDateTime(datetimestring, formats='ymd mdy dmy'))
        Return value is based on date and (if any) time data in the string.
    DateTime = numericDate(datestring, formats='ymd mdy dmy'))
        Return value is based on date data in the string.
    DateTimeDelta = numericTime(timestring)
        Return value is based on time data in the string.

20010804 JJD Created, borrowing some regular expressions and
         ideas from mxDateTime's Parser module.
20020404 JJD Fixed _THIS_CENTURY to use 100-year century (not 1000).

Donated to eGenix for mxDateTime
"""
from mx import DateTime
import re

# RE's can be simple if the dates are all-numeric
# A year, month, or day is some number of digits not preceded by
# or followed by a digit or a colon.
_year  = r'(?P<year>(?<![\d:])\d{2,4}(?![\d:]))'
_month = r'(?P<month>(?<![\d:])\d{1,2}(?![\d:]))'
_day   = r'(?P<day>(?<![\d:])\d{1,2}(?![\d:]))'
# The <sep> group is used in a way to guarantee that the same
# separator character is used twice in the date expression.
_ymd = _year  + r'(?P<sep>\D)' + _month + r'(?P=sep)' + _day + r'(?!\d)'
_dmy = _day   + r'(?P<sep>\D)' + _month + r'(?P=sep)' + _year + r'(?!\d)'
_mdy = _month + r'(?P<sep>\D)' + _day   + r'(?P=sep)' + _year + r'(?!\d)'
_ymdRE  = re.compile(_ymd,  re.I)
_dmyRE  = re.compile(_dmy,  re.I)
_mdyRE  = re.compile(_mdy,  re.I)
_formatdict = {'ymd':_ymdRE, 'dmy':_dmyRE, 'mdy':_mdyRE}

_hour = r'(?P<hour>[012]?\d)'
_minute = r'(?P<minute>[0-6]\d)'
_second = r'(?P<second>[0-6]\d(?:\.\d+)?)'
_time = _hour + r':' + _minute + r'(?::' + _second + r')?'
_timeRE = re.compile(_time, re.I)

# Use divmod to avoid Guido's evil plot to break integer division
_THIS_CENTURY = 100*divmod(DateTime.now().year, 100)[0]

def _iso_add_century(year):
    """
    According to ISO 8601, a two-digit year in an all-numeric date
    expression refers to the current century.
    If year has exactly two digits, add the century. Otherwise, return
    it unchanged.
    """
    if 0 <= year < 100:
        result = year + _THIS_CENTURY
    else:
        result = year
    return result

def numericDate(s, formats='ymd mdy dmy'):
    """
    If the input string s is a valid numeric date
    expression according to one of the formats given, return
    a DateTime object with default time components.
    Date formats, separated by a single space character, are
    packed into a string.

    If only one date format is given, raise a ValueError if
    the string cannot be interpreted, and raise a DateTime
    RangeError if it can be interpreted but is not a valid date.

    If more than one date format is given, try each of them in order
    until a valid interpretation is found.  If the string does
    not represent a valid date, return None.  You can use one
    format twice (e.g. ['ymd', 'ymd']) to test only one format but
    suppress exceptions.
    """
    if s is None: return None
    dt = None
    formatlist = formats.split()
    exceptions_ok = (len(formatlist) == 1)
    _s = str(s).strip()
    len_s = len(_s)
    if (len_s in (6, 8)) and s.isdigit():
        # Accept yyyymmdd or yymmdd
        k0 = len_s - 4; k1 = k0 + 2; k2 = k1 + 2
        yy = _iso_add_century(int(_s[0:k0]))
        mm = int(_s[k0:k1])
        dd = int(_s[k1:k2])
        try:
            dt = DateTime.DateTime(yy, mm, dd)
        except DateTime.RangeError, why:
            dt = None
            if exceptions_ok:
                raise DateTime.RangeError, "'%s' invalid: %s" % (_s, why)
    else:
        # Try the given formats in order
        for format in formatlist:
            format_re = _formatdict[format.lower()]
            match = format_re.search(_s)
            if match is None:
                continue
            year = match.group('year')
            month = match.group('month')
            day = match.group('day')
            yy = _iso_add_century(int(year))
            mm = int(month)
            dd = int(day)
            try:
                dt = DateTime.DateTime(yy, mm, dd)
            except DateTime.RangeError, why:
                dt = None
                if exceptions_ok:
                    raise DateTime.RangeError, "'%s' invalid: %s" % (_s, why)
            if dt is not None:
                break
        if dt is None and exceptions_ok:
            raise ValueError, "unrecognized date format: '%s'" % _s
    return dt

def numericTime(s):
    """
    If the input string s is a valid time expression of the
    form hh:mm:ss.sss or hh:mm:ss or hh:mm, return
    a corresponding DateTimeDelta object. Otherwise, return None.
    """
    if s is None: return None
    dt = None
    match = _timeRE.search(s)
    if match is not None:
        hour = match.group('hour')
        minute = match.group('minute')
        second = match.group('second')
        hh = int(hour); mm = int(minute)
        if second:
            ss = float(second)
        else:
            ss = 0.0
        try:
            dt = DateTime.DateTimeDelta(0, hh, mm, ss)
        except DateTime.RangeError:
            dt = None
    return dt

def numericDateTime(s, formats='ymd mdy dmy'):
    # Return a DateTime object or None, incorporating time data
    # if included in the string. If only one format is given,
    # and if the string is not a valid date-time expression
    # according to that format, raise an exception.
    if s is None: return None
    # ISO date-time may have T separating date and time
    _s = str(s).strip().replace('T', ' ')
    date_time = _s.split()
    d = numericDate(date_time[0], formats)
    if d is not None and len(date_time) > 1:
        t = numericTime(date_time[1])
        if t is not None:
            d = d + t
    return d

def _test_it(testcase, formats, timeonly=0):
    result = None
    if timeonly:
        try:
            dtd0 = numericTime(testcase[0])
            if dtd0 != testcase[1]:
                result = "Error: '%s' --> %s" % (testcase[0], repr(dtd0))
        except Exception, e:
            result = "Error: '%s' -- %s" % (testcase[0], str(e))
    else:
        try:
            dt0 = numericDateTime(testcase[0], formats)
            if dt0 != testcase[1]:
                result = "Error: '%s' --> %s" % (testcase[0], repr(dt0))
        except Exception, e:
            result = "Error: '%s' -- %s" % (testcase[0], str(e))
    return result

def test(formats='ymd mdy dmy'):
    from time import clock
    testcases = [ \
        # Time expressions
        ("12:13:14.56", DateTime.DateTimeDelta(0, 12, 13, 14.56)),
        ("12:13:14", DateTime.DateTimeDelta(0, 12, 13, 14)),
        ("12:13", DateTime.DateTimeDelta(0, 12, 13, 0)),
        # Date or date-time expressions (mostly)
        ("Ceci n'est pas une date", None),
        ("010803", DateTime.DateTime(2001, 8, 3)),
        ("20010803", DateTime.DateTime(2001, 8, 3)),
        ("20010803 12:13:14.56", DateTime.DateTime(2001, 8, 3, 12, 13, 14.56)),
        ("20010803T12:13:14.56", DateTime.DateTime(2001, 8, 3, 12, 13, 14.56)),
        ("01-08-03", DateTime.DateTime(2001, 8, 3)),
        ("2001-08-03", DateTime.DateTime(2001, 8, 3)),
        ("2001-8-3", DateTime.DateTime(2001, 8, 3)),
        ("2001-08-03 12:13:14.56", DateTime.DateTime(2001, 8, 3, 12, 13, 14.56)),
        ("2001-08-03T12:13:14.56", DateTime.DateTime(2001, 8, 3, 12, 13, 14.56)),
        # These should be correctly recognized if tested as mdy.
        ("08/03/2001", DateTime.DateTime(2001, 8, 3)),
        ("8/3/2001", DateTime.DateTime(2001, 8, 3)),
        ("08/03/2001 12:13:14.56", DateTime.DateTime(2001, 8, 3, 12, 13, 14.56)),
        ("08/03/2001T12:13:14.56", DateTime.DateTime(2001, 8, 3, 12, 13, 14.56)),
        # These can only be mdy
        ("08/23/2001", DateTime.DateTime(2001, 8, 23)),
        ("8/23/2001", DateTime.DateTime(2001, 8, 23)),
        ("08/23/2001 12:13:14.56", DateTime.DateTime(2001, 8, 23, 12, 13, 14.56)),
        # These can only be dmy
        ("23.08.2001", DateTime.DateTime(2001, 8, 23)),
        ("23.8.2001", DateTime.DateTime(2001, 8, 23)),
        ("23.08.2001 12:13:14.56", DateTime.DateTime(2001, 8, 23, 12, 13, 14.56)),
        ]
    print "\nTesting numdate.py with format(s) %s" % ", ".join(formats.split())
    t0 = clock()
    cases = 0
    for i in range(100):
        for testcase in testcases[0:3]:
            result = _test_it(testcase, formats, timeonly=1)
            cases = cases + 1
            if result and i == 0:
                print result

        for testcase in testcases[3:]:
            result = _test_it(testcase, formats, timeonly=0)
            cases = cases + 1
            if result and i == 0:
                print result
    t1 = clock()
    print "Done - %d cases tested in %1.5f sec." %  (cases, t1 - t0)

if __name__ == '__main__':
    test()



""" NewBuiltins - Installs (most of) the mx.Tools add-ons for Python
                  as Python builtins or in the sys module.

    The installation is done upon import, so that you only need
    to include the line 'import mx.Tools.NewBuiltins' at the top of your
    script to have the add-ons available for use in the script.

    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
import sys,types,operator
import Tools

# Optional intra-package tools
try:
    from mx.DateTime import DateTimeFrom
except ImportError:
    DateTimeFrom = None

# Enable debugging output
_debug = 0

# Helper
def install_objects(namespace, objects,

                    DictType=types.DictType):

    """ Install all the given objects in namespace.

        Doesn't overwrite anything already defined in namespace.
        
    """
    for obj in objects:
        try:
            name = obj.__name__
        except AttributeError:
            try:
                name, obj = obj
            except (TypeError, ValueError):
                name = repr(obj)
        if name[:2] != '__':
            if namespace.has_key(name):
                if _debug:
                    try:
                        import warnings
                    except ImportError:
                        pass
                    else:
                        warnings.warn('mxTools builtin %s would overwrite existing '
                                      'builtin; not installed' % name,
                                      RuntimeWarning)
            else:
                namespace[name] = obj

### Note that undocumented functions may well disappear in
### future releases !!!

# New APIs and objects that go into __builtins__
install_objects(__builtins__,
                (Tools.NotGiven,
                 ('True', Tools.True),	# already defined in Python >= 2.2.x !
                 ('False', Tools.False),# already defined in Python >= 2.2.x !
                 Tools.acquire, 
                 Tools.attrlist,
                 Tools.count, 
                 Tools.defined,
                 Tools.dict,		# already defined in Python >= 2.2.x !
                 Tools.exists,
                 Tools.extract,
                 Tools.findattr,
                 Tools.forall, 
                 Tools.frange,          # undocumented
                 Tools.get, 
                 Tools.ifilter,
                 Tools.index, 
                 Tools.indices, 
                 Tools.invdict,
                 Tools.irange, 
                 Tools.iremove,
                 Tools.issequence,      # undocumented
                 Tools.lists, 
                 Tools.mapply,
                 Tools.method_mapply, 
                 #Tools.mget,           # old
                 #Tools.mgetattr,       # old
                 Tools.napply,
                 #Tools.optimization,   # moved to sys
                 Tools.projection,      # undocumented
                 Tools.range_len,
                 Tools.reval,
                 Tools.reverse, 
                 Tools.setdict,
                 Tools.sign,
                 Tools.sizeof, 
                 Tools.sortedby,        # undocumented
                 Tools.trange,
                 #Tools.trange_len,     # old
                 Tools.truth,
                 ('boolean', Tools.truth), # defined as bool() in Python >= 2.2.x !
                 Tools.tuples,
                 #Tools.verbosity,      # moved to sys
                 #Tools.xmap,           # no longer supported
                 ('binary', buffer),
                 )
                )

# Optional additional builtins
if DateTimeFrom is not None:
    install_objects(__builtins__, (('datetime', DateTimeFrom),))

# New APIs for the sys module
install_objects(sys.__dict__,
                (Tools.optimization, 
                 Tools.verbosity,
                 Tools.debugging,
                 Tools.interactive,
                 Tools.cur_frame,
                 Tools.makeref,
                 )
                )

# All other APIs are available through the Tools package, e.g
# Tools.docstring, etc.



#!/usr/local/bin/python2.1

""" Configuration for the eGenix mx Base Distribution

    Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
import sys
from mxSetup import mx_Extension, mx_version

#
# Package version
#
version = mx_version(3, 2, 7)

#
# Setup information
#
name = "egenix-mx-base"

#
# Meta-Data
#
description = "eGenix mx Base Distribution for Python - mxDateTime, mxTextTools, mxProxy, mxTools, mxBeeBase, mxStack, mxQueue, mxURL, mxUID"
long_description = """\
The eGenix mx Extension Series are a collection of Python extensions
written in ANSI C and Python which provide a large spectrum of useful
additions to everyday Python programming.

The Base Distribution includes the Open Source subpackages of the
series and is needed by all other add-on packages of the series:

mxDateTime - Date/Time Library for Python

    mxDateTime implements three new object types, DateTime,
    DateTimeDelta and RelativeDateTime and many tools based on these
    for doing easy conversion between and parsing of various date/time
    formats.

mxTextTools - Fast Text Parsing and Processing Tools for Python

    mxTextTools provides several useful functions and types that
    implement high-performance text parsing, processing and search
    algorithms.

mxProxy - Object Access Control for Python

    mxProxy implements a new proxy type to provide low-level object
    access control, weak referencing and a cleanup protocol. It's
    ideal for use in restricted execution environments.

mxBeeBase - On-disk B+Tree Based Database Kit for Python

    mxBeeBase is a high performance construction kit for disk based
    indexed databases. It offers components which you can plug
    together to easily build your own custom mid-sized databases.

mxURL - Flexible URL Data-Type for Python

    mxURL provides a new datatype for storing and manipulating URL
    values as well as a few helpers related to URL building, encoding
    and decoding.

mxUID - Fast Universal Identifiers for Python

    mxUID implements a fast mechanism for generating universal
    identification strings (UIDs).

mxStack - Fast and Memory-Efficient Stack Type for Python

    mxStack implements a fast and memory efficient stack object type.

mxQueue - Fast and Memory-Efficient Queue Type for Python

    mxQueue implements a fast and memory efficient queue object type.

mxTools - Fast Everyday Helpers for Python

    mxTools provides a collection of handy functions and objects for
    every day Python programming. It includes many functions that
    you've often missed in Python.

This software is brought to you by eGenix.com and distributed under
the eGenix.com Public License 1.1.0.
"""
license = (
"eGenix.com Public License 1.1.0; "
"Copyright (c) 1997-2000, Marc-Andre Lemburg, All Rights Reserved; "
"Copyright (c) 2000-2013, eGenix.com Software GmbH, All Rights Reserved"
)
author = "eGenix.com Software GmbH"
author_email = "info@egenix.com"
maintainer = "eGenix.com Software GmbH"
maintainer_email = "info@egenix.com"
url = "http://www.egenix.com/products/python/mxBase/"
download_url = 'https://downloads.egenix.com/python/download_url/%s/%s/' % (
    name,
    version)
platforms = [
    'Windows',
    'Linux',
    'FreeBSD',
    'Solaris',
    'Mac OS X',
    'AIX',
    ]
classifiers = [
    "Environment :: Console",
    "Environment :: No Input/Output (Daemon)",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: Python License (CNRI Python License)",
    "License :: Freely Distributable",
    "License :: Other/Proprietary License",
    "Natural Language :: English",
    "Operating System :: OS Independent",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: POSIX",
    "Operating System :: Unix",
    "Operating System :: BeOS",
    "Operating System :: MacOS",
    "Operating System :: OS/2",
    "Operating System :: Other OS",
    "Programming Language :: C",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.4",
    "Programming Language :: Python :: 2.5",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Topic :: Communications",
    "Topic :: Database",
    "Topic :: Documentation",
    "Topic :: Internet",
    "Topic :: Internet :: WWW/HTTP",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: CGI Tools/Libraries",
    "Topic :: Internet :: WWW/HTTP :: Site Management :: Link Checking",
    "Topic :: Scientific/Engineering",
    "Topic :: Scientific/Engineering :: Interface Engine/Protocol Translator",
    "Topic :: Software Development",
    "Topic :: Software Development :: Libraries",
    "Topic :: Software Development :: Libraries :: Application Frameworks",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Text Processing",
    "Topic :: Text Processing :: Filters",
    "Topic :: Text Processing :: Markup",
    "Topic :: Utilities ",
    ]
if 'a' in version:
    classifiers.append("Development Status :: 3 - Alpha")
elif 'b' in version:
    classifiers.append("Development Status :: 4 - Beta")
else:
    classifiers.append("Development Status :: 5 - Production/Stable")
    classifiers.append("Development Status :: 6 - Mature")
classifiers.sort()

#
# Python packages
#
packages = [

    # mx Extensions Base Package
    'mx',

    # mxDateTime
    'mx.DateTime',
    'mx.DateTime.mxDateTime',
    'mx.DateTime.Examples',

    # mxProxy
    'mx.Proxy',
    'mx.Proxy.mxProxy',

    # mxQueue
    'mx.Queue',
    'mx.Queue.mxQueue',

    # mxStack
    'mx.Stack',
    'mx.Stack.mxStack',

    # mxTextTools
    'mx.TextTools',
    'mx.TextTools.mxTextTools',
    'mx.TextTools.Constants',
    'mx.TextTools.Examples',

    # mxTools
    'mx.Tools',
    'mx.Tools.mxTools',
    'mx.Tools.Examples',

    # mxBeeBase
    'mx.BeeBase',
    'mx.BeeBase.mxBeeBase',

    # mxURL
    'mx.URL',
    'mx.URL.mxURL',
    
    # mxUID
    'mx.UID',
    'mx.UID.mxUID',

    # Misc. other modules
    'mx.Misc',

    ]

#
# C Extensions
#

# Determine optional platform-dependent features
if sys.platform[:3] != 'win':
    # Unix-like platforms
    _mxDateTime_optional_libraries = [
        # mxDateTime needs floor() and ceil() which are sometimes
        # defined in libm. Python normally already references this
        # library if necessary, so not finding the library is not
        # necessarily a reason to fail building mxDateTime.
        ('m', ['math.h']),
        # mxDateTime can use the API clock_gettime() if available,
        # but this sometimes needs the librt to be available.
        ('rt', ['time.h']),
        ]
    _mxTools_optional_libraries = [
        # The optional mx.Tools.dlopen() function needs the dl lib in
        # order to load dynamic libaries on Unix platforms.
        ('dl', ['dlfcn.h']),
        ]
else:
    # On Windows, the extra libs are not needed (or even available)
    _mxDateTime_optional_libraries = []
    _mxTools_optional_libraries = []
    
# Extension definitions
ext_modules = [

    # mxDateTime
    mx_Extension('mx.DateTime.mxDateTime.mxDateTime',
                 ['mx/DateTime/mxDateTime/mxDateTime.c'],
                 # If mxDateTime doesn't compile, try removing the next line.
                 define_macros=[('USE_FAST_GETCURRENTTIME', None)],
                 #
                 include_dirs=['mx/DateTime/mxDateTime'],
                 optional_libraries=_mxDateTime_optional_libraries,
                 ),

    # mxProxy
    mx_Extension('mx.Proxy.mxProxy.mxProxy',
                 ['mx/Proxy/mxProxy/mxProxy.c'],
                 include_dirs=['mx/Proxy/mxProxy']),

    # mxQueue
    mx_Extension('mx.Queue.mxQueue.mxQueue',
                 ['mx/Queue/mxQueue/mxQueue.c'],
                 include_dirs=['mx/Queue/mxQueue']),

    # mxStack
    mx_Extension('mx.Stack.mxStack.mxStack',
                 ['mx/Stack/mxStack/mxStack.c'],
                 include_dirs=['mx/Stack/mxStack']),

    # mxTextTools
    mx_Extension('mx.TextTools.mxTextTools.mxTextTools',
                 ['mx/TextTools/mxTextTools/mxTextTools.c',
                  'mx/TextTools/mxTextTools/mxte.c',
                  'mx/TextTools/mxTextTools/mxbmse.c'],
                 define_macros=[('MX_BUILDING_MXTEXTTOOLS', None)],
                 include_dirs=['mx/TextTools/mxTextTools']),

    # mxTools
    mx_Extension('mx.Tools.mxTools.mxTools',
                 ['mx/Tools/mxTools/mxTools.c'],
                 define_macros=[

                     # To enable mx.Tools.setproctitle(), you have to enable
                     # the following line. Note that not all Python versions
                     # expose the required Py_GetArgcArgv() API.
                     # ('HAVE_PY_GETARGCARGV', None),
                     
                     # If you want to use the experimental mx.Tools.safecall()
                     # API, you have to enable the following line.
                     # ('MXTOOLS_ENABLE_SAFECALL', None),
                     
                 ],
                 include_dirs=['mx/Tools/mxTools'],
                 optional_libraries=_mxTools_optional_libraries,
                 ),

    # xmap is no longer supported
    #mx_Extension('mx.Tools.mxTools.xmap',
    #             ['mx/Tools/mxTools/xmap.c'],
    #             include_dirs=['mx/Tools/mxTools']),

    # mxBeeBase
    mx_Extension('mx.BeeBase.mxBeeBase.mxBeeBase',
                 ['mx/BeeBase/mxBeeBase/mxBeeBase.c',
                  'mx/BeeBase/mxBeeBase/btr.c'],
                 include_dirs=['mx/BeeBase/mxBeeBase']),

    # mxURL
    mx_Extension('mx.URL.mxURL.mxURL',
                 ['mx/URL/mxURL/mxURL.c'],
                 include_dirs=['mx/URL/mxURL']),

    # mxUID
    mx_Extension('mx.UID.mxUID.mxUID',
                 ['mx/UID/mxUID/mxUID.c'],
                 include_dirs=['mx/UID/mxUID']),

    ]

#
# Data files
#
data_files = [

    # Copyright, licenses, READMEs
    'mx/COPYRIGHT',
    'mx/LICENSE',
    'mx/README',

    # Misc
    'mx/Misc/LICENSE',
    'mx/Misc/COPYRIGHT',

    # mxDateTime
    'mx/DateTime/Doc/mxDateTime.pdf',
    'mx/DateTime/COPYRIGHT',
    'mx/DateTime/LICENSE',
    'mx/DateTime/README',
    'mx/DateTime/mxDateTime/mxDateTime.h',
    'mx/DateTime/mxDateTime/mxh.h',

    # mxProxy
    'mx/Proxy/Doc/mxProxy.pdf',
    'mx/Proxy/COPYRIGHT',
    'mx/Proxy/LICENSE',
    'mx/Proxy/README',
    'mx/Proxy/mxProxy/mxProxy.h',
    'mx/Proxy/mxProxy/mxh.h',

    # mxQueue
    'mx/Queue/Doc/mxQueue.pdf',
    'mx/Queue/COPYRIGHT',
    'mx/Queue/LICENSE',
    'mx/Queue/README',
    'mx/Queue/mxQueue/mxQueue.h',
    'mx/Queue/mxQueue/mxh.h',

    # mxStack
    'mx/Stack/Doc/mxStack.pdf',
    'mx/Stack/COPYRIGHT',
    'mx/Stack/LICENSE',
    'mx/Stack/README',
    'mx/Stack/mxStack/mxStack.h',
    'mx/Stack/mxStack/mxh.h',

    # mxTextTools
    'mx/TextTools/Doc/mxTextTools.pdf',
    'mx/TextTools/COPYRIGHT',
    'mx/TextTools/LICENSE',
    'mx/TextTools/README',
    'mx/TextTools/mxTextTools/mxTextTools.h',
    'mx/TextTools/mxTextTools/mxh.h',
    'mx/TextTools/mxTextTools/mxbmse.h',
    
    # mxTools
    'mx/Tools/Doc/mxTools.pdf',
    'mx/Tools/COPYRIGHT',
    'mx/Tools/LICENSE',
    'mx/Tools/README',
    'mx/Tools/mxTools/mxTools.h',
    'mx/Tools/mxTools/mxh.h',

    # mxBeeBase
    'mx/BeeBase/Doc/mxBeeBase.pdf',
    'mx/BeeBase/COPYRIGHT',
    'mx/BeeBase/LICENSE',
    'mx/BeeBase/README',
    'mx/BeeBase/mxBeeBase/mxBeeBase.h',
    'mx/BeeBase/mxBeeBase/mxh.h',
    'mx/BeeBase/mxBeeBase/btr.h',

    # mxURL
    'mx/URL/Doc/mxURL.pdf',
    'mx/URL/COPYRIGHT',
    'mx/URL/LICENSE',
    'mx/URL/README',
    'mx/URL/mxURL/mxURL.h',
    'mx/URL/mxURL/mxh.h',

    # mxUID
    'mx/UID/Doc/mxUID.pdf',
    'mx/UID/COPYRIGHT',
    'mx/UID/LICENSE',
    'mx/UID/README',
    'mx/UID/mxUID/mxUID.h',
    'mx/UID/mxUID/mxh.h',

    ]

# Declare namespace packages (for building eggs)
namespace_packages = [
    'mx',
    ]



from mx.DateTime import now, TimeDelta

if 0:
    # Force import of the datetime module
    d = now()
    d.pydate()

td = TimeDelta(3)
print td
td1 = td.rebuild()
print td1
pytime = td.pytime()
print pytime
pydelta = td.pytimedelta()
print pydelta



""" Python part of the low-level DateTime[Delta] type implementation.

    Copyright (c) 1998-2001, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
# Import C extension module
from mxDateTime import *
from mxDateTime import __version__

# Singletons
oneSecond = DateTimeDelta(0,0,0,1)
oneMinute = DateTimeDelta(0,0,1)
oneHour = DateTimeDelta(0,1)
oneDay = DateTimeDelta(1)
oneWeek = DateTimeDelta(7)
Epoch = DateTimeFromAbsDateTime(1, 0)

# Shortcuts for pickle; for backward compatibility only (they are now
# defined in __init__.py to further reduce the pickles length)
def _DT(absdate,abstime):
    return DateTimeFromAbsDateTime(absdate,abstime)
def _DTD(seconds):
    return DateTimeDeltaFromSeconds(seconds)

# Module init
class modinit:

    global _time, _math, _types, _sys, _pydatetime, _pydate, _pytime, _pytimedelta
    import time, math, types, sys
    _time = time
    _math = math
    _types = types
    _sys = sys
    try:
        import datetime
        _pydatetime = datetime.datetime
        _pydate = datetime.date
        _pytime = datetime.time
        _pytimedelta = datetime.timedelta
    except ImportError:
        # Set them to (), so that isinstance() works with them
        _pydatetime = ()
        _pydate = ()
        _pytime = ()
        _pytimedelta = ()

del modinit

### Helpers

def _isstring(arg,

              isinstance=isinstance, types=_types):
    
    if isinstance(arg, types.StringType):
        return 1
    try:
        if isinstance(arg, types.UnicodeType):
            return 1
    except AttributeError:
        pass
    return 0

### Compatibility APIs

# Aliases and functions to make 'from mx.DateTime import *' work much
# like 'from time import *'

def localtime(ticks=None,
              # Locals:
              time=_time.time,float=float,localtime=_time.localtime,
              round=round,int=int,DateTime=DateTime,floor=_math.floor):

    """localtime(ticks=None)

       Construct a DateTime instance using local time from ticks.  If
       ticks are not given, it defaults to the current time.  The
       result is similar to time.localtime(). Fractions of a second
       are rounded to the nearest micro-second.

    """
    if ticks is None:
        ticks = time()
    else:
        ticks = float(ticks)
    ticks = round(ticks, 6)
    fticks = floor(ticks)
    Y,M,D,h,m,s = localtime(fticks)[:6]
    s = s + (ticks - fticks)
    return DateTime(Y,M,D,h,m,s)

def gmtime(ticks=None,
           # Locals:
           time=_time.time,float=float,gmtime=_time.gmtime,
           round=round,int=int,DateTime=DateTime,floor=_math.floor,
           DateTimeDeltaFromSeconds=DateTimeDeltaFromSeconds,
           Unix_Epoch=DateTime(1970, 1, 1)):

    """gmtime(ticks=None)

       Construct a DateTime instance using UTC time from ticks.  If
       ticks are not given, it defaults to the current time.  The
       result is similar to time.gmtime(). Fractions of a second are
       rounded to the nearest micro-second.

    """
    if ticks is None:
        ticks = time()
    else:
        ticks = float(ticks)
    ticks = round(ticks, 6)
    if POSIX:
        # On POSIX platforms, the calulation is easy (and this also
        # makes the calulation safe against year 2038 wrap-arounds and
        # errors); see #421
        return Unix_Epoch + DateTimeDeltaFromSeconds(float(ticks))
    fticks = floor(ticks)
    Y,M,D,h,m,s = gmtime(ticks)[:6]
    s = s + (ticks - fticks)
    return DateTime(Y,M,D,h,m,s)

def mktime((year,month,day,hour,minute,second,dow,doy,dst),
           # Locals:
           DateTime=DateTime):

    """mktime((year,month,day,hour,minute,second,dow,doy,dst))

       Same as the DateTime() constructor accept that the interface
       used is compatible to the similar time.mktime() API.

       Note that the tuple elements dow, doy and dst are not used in
       any way.
      
    """
    return DateTime(year,month,day,hour,minute,second)

def ctime(datetime):

    """ctime(datetime)

       Returns a string representation of the given DateTime instance
       using the current locale's default settings.

    """
    return datetime.strftime('%c')

def today(hour=0,minute=0,second=0.0,
          # Locals:
          localtime=_time.localtime,time=_time.time,DateTime=DateTime):

    """today(hour=0,minute=0,second=0.0)

       Returns a DateTime instance for today (in local time) at the
       given time (defaults to midnight).

    """
    Y,M,D = localtime(time())[:3]
    return DateTime(Y,M,D,hour,minute,second)

def TimeDelta(hours=0.0,minutes=0.0,seconds=0.0,
              # Locals:
              DateTimeDelta=DateTimeDelta):

    """TimeDelta(hours=0.0,minutes=0.0,seconds=0.0)

       Returns a DateTimeDelta-object reflecting the given time
       delta. Seconds can be given as float to indicate fractions.

    """
    return DateTimeDelta(0,hours,minutes,seconds)

def gm2local(datetime):

    """ gm2local(datetime)

        Convert a DateTime instance holding UTC time to a DateTime
        instance using local time.

    """
    return localtime(datetime.gmticks())

def local2gm(datetime):

    """ local2gm(datetime)

        Convert a DateTime instance holding local time to a DateTime
        instance using UTC time.

    """
    return gmtime(datetime.ticks())

# Alias
gmt = utc

# Default value for DateTimeFromTJD's tjd_myriad parameter
current_myriad = localtime().tjd_myriad

def DateTimeFromTJD(tjd,tjd_myriad=current_myriad):

    """ DateTimeFromTJD(tjd[,myriad])

        Return a DateTime instance for the given Truncated Julian Day.
        myriad defaults to the TJD myriad current at package import
        time.

        Note that this version of Truncated Julian Day number does
        real truncation of important information. It's use is
        discouraged and unsupported.

    """
    return DateTimeFromAbsDays(tjd + tjd_myriad * 10000.0 - 1721425.0)

def DateTimeFromJDN(jdn):

    """ DateTimeFromJDN(jdn)

        Return a DateTime instance for the given Julian Day Number.

        References:
        -----------
        Gregorian 2000-01-01 12:00:00 corresponds to JDN 2451545.0.
        Gregorian 1858-11-17 00:00:00.00 corresponds to JDN 2400000.5; MJD 0.0.
        Julian -4712-01-01 12:00:00.00 corresponds to JDN 0.0.
        Gregorian -4713-11-24 12:00:00.00 corresponds to JDN 0.0.

    """
    return DateTimeFromAbsDays(jdn - 1721425.5)

def DateTimeFromMJD(mjd):

    """ DateTimeFromMJD(mjd)

        Return a DateTime instance for the given Modified Julian Day
        (MJD). The MJD is calculated the same way as the JDN except
        that 1858-11-17 00:00:00.00 is taken as origin of the scale.

    """
    return DateTimeFromAbsDays(mjd + 678575.0)

def DateTimeFrom(*args, **kws):

    """ DateTimeFrom(*args, **kws)

        Generic DateTime instance constructor. Can handle parsing
        strings, numbers and keywords.

    """
    if len(args) == 1:
        # Single argument
        arg = args[0]
        argtype = type(arg)
        if _isstring(arg):
            import Parser
            return Parser.DateTimeFromString(*args, **kws)
        elif argtype is DateTimeType:
            return arg
        elif argtype is DateTimeDeltaType:
            raise TypeError,'cannot convert DateTimeDelta to DateTime'
        elif isinstance(arg, _pydatetime):
            if arg.tzinfo is not None:
                # Convert to UTC
                offset = arg.utcoffset()
                arg -= offset
            return DateTime(
                arg.year, arg.month, arg.day,
                arg.hour, arg.minute,
                arg.second + arg.microsecond * 1e-6)
        elif isinstance(arg, _pydate):
            # Note: _pydatetime are subclasses of _pydates !
            return DateTime(arg.year, arg.month, arg.day)
        else:
            try:
                value = float(arg)
            except (TypeError, ValueError):
                value = int(arg)
            assert not kws
            return DateTimeFromTicks(value)

    elif len(args) > 1:
        # More than one argument
        if len(args) == 2 and _isstring(args[0]) and _isstring(args[1]):
            # interpret as date and time string
            import Parser
            return Parser.DateTimeFromString(
                args[0] + ' ' + args[1],
                **kws)

        # Assume the arguments are the same as for DateTime()
        if 'defaultdate' in kws:
            raise TypeError('defaultdate is only supported when parsing '
                            'strings or using keyword arguments only')
        return DateTime(*args, **kws)

    elif len(kws) > 0:
        # Keyword arguments only; add defaults... today at 0:00:00
        defaultdate = kws.get('defaultdate', None)
        if defaultdate is None:
            defaultdate = today()
        hour = kws.get('hour', defaultdate.hour)
        minute = kws.get('minute', defaultdate.minute)
        second = kws.get('second', defaultdate.second)
        day = kws.get('day', defaultdate.day)
        month = kws.get('month', defaultdate.month)
        year = kws.get('year', defaultdate.year)
        return DateTime(year,month,day,hour,minute,second)

    else:
        raise TypeError(
            'cannot convert date/time arguments to DateTime: '
            'args=%r, kws=%r' % (args, kws))

def DateFrom(*args, **kws):

    """ DateFrom(*args, **kws)

        Generic DateTime instance constructor. Can handle parsing
        strings, numbers and keywords, but only works on the date
        part.

    """
    if len(args) == 1:
        # Single argument
        arg = args[0]
        argtype = type(arg)
        if _isstring(arg):
            import Parser
            return Parser.DateFromString(*args, **kws)
        elif argtype is DateTimeType:
            return arg
        elif argtype is DateTimeDeltaType:
            raise TypeError,'cannot convert DateTimeDelta to DateTime'
        elif isinstance(arg, _pydate):
            # Note: _pydatetime are subclasses of _pydates !
            return DateTime(arg.year, arg.month, arg.day)
        else:
            try:
                value = float(arg)
            except (TypeError, ValueError):
                value = int(arg)
            assert not kws
            return DateTimeFromTicks(value)

    elif len(args) > 1:
        # Assume the arguments are the same as for DateTime()
        return DateTime(*args, **kws)

    elif len(kws) > 0:
        # Keyword arguments; add defaults... today at 0:00:00
        today = now()
        day = kws.get('day',today.day)
        month = kws.get('month',today.month)
        year = kws.get('year',today.year)
        return DateTime(year,month,day)

    else:
        raise TypeError(
            'cannot convert date arguments to DateTime: args=%r, kws=%r' %
            (args, kws))

def DateTimeDeltaFrom(*args, **kws):

    """ DateTimeDeltaFrom(*args, **kws)

        Generic DateTimeDelta instance constructor. Can handle parsing
        strings, numbers and keywords.

    """
    if len(args) == 1:
        # Single argument
        arg = args[0]
        if _isstring(arg):
            import Parser
            return Parser.DateTimeDeltaFromString(*args, **kws)
        elif type(arg) is DateTimeDeltaType:
            return arg
        elif type(arg) is DateTimeType:
            raise TypeError,'cannot convert DateTime to DateTimeDelta'
        elif isinstance(arg, _pytime):
            if arg.tzinfo is not None:
                # Convert to UTC
                offset = arg.utcoffset()
                # Another datetime module quirk: time - timedelta
                # doesn't work, so we have to use a different method
                # of adjusting the result
                return DateTimeDelta(0,
                                     arg.hour, arg.minute,
                                     arg.second + arg.microsecond * 1e-6
                                     - offset.seconds)
            else:
                return DateTimeDelta(0,
                                     arg.hour, arg.minute,
                                     arg.second + arg.microsecond * 1e-6)
        elif isinstance(arg, _pytimedelta):
            return DateTimeDelta(arg.days,
                                 0, 0,
                                 arg.seconds + arg.microseconds * 1e-6)
        else:
            try:
                value = float(arg)
            except TypeError:
                value = int(arg)
            assert not kws
            return DateTimeDeltaFromSeconds(value)

    elif len(args) > 1:
        # Assume the arguments are the same as for DateTimeDelta()
        return DateTimeDelta(*args, **kws)

    elif len(kws) > 0:
        # Keyword arguments; default: 00:00:00:00.00
        hours = kws.get('hours',0)
        minutes = kws.get('minutes',0)
        seconds = kws.get('seconds',0.0)
        days = kws.get('days',0)
        return DateTimeDelta(days,hours,minutes,seconds)

    else:
        raise TypeError,'cannot convert arguments to DateTimeDelta'

def TimeDeltaFrom(*args, **kws):

    """ TimeDeltaFrom(*args, **kws)

        Generic TimeDelta instance constructor. Can handle parsing
        strings, numbers and keywords.

    """
    if len(args) > 1:
        # Assume the arguments are the same as for TimeDelta(): without
        # days part !
        return DateTimeDelta(0, *args, **kws)
    else:
        # Otherwise treat the arguments just like for DateTimeDelta
        # instances.
        return DateTimeDeltaFrom(*args, **kws)

def DateFromTicks(ticks,
                  # Locals:
                  DateTime=DateTime,localtime=_time.localtime):

    """ DateFromTicks(ticks)

        Constructs a DateTime instance pointing to the local time date
        at 00:00:00.00 (midnight) indicated by the given ticks value.
        The time part is ignored.

    """
    return DateTime(*localtime(ticks)[:3])

def TimestampFromTicks(ticks,
                       # Locals:
                       DateTime=DateTime,localtime=_time.localtime):

    """ TimestampFromTicks(ticks)

        Constructs a DateTime instance pointing to the local date and
        time indicated by the given ticks value.

    """
    return DateTime(*localtime(ticks)[:6])

def TimeFromTicks(ticks,
                  # Locals:
                  DateTimeDelta=DateTimeDelta,localtime=_time.localtime):

    """ TimeFromTicks(ticks)

        Constructs a DateTimeDelta instance pointing to the local time
        indicated by the given ticks value. The date part is ignored.

    """
    return DateTimeDelta(0, *localtime(ticks)[3:6])

# Aliases
utctime = gmtime
utc2local = gm2local
local2utc = local2gm
DateTimeFromTicks = localtime
Date = DateTime
Time = TimeDelta
Timestamp = DateTime
TimeFrom = TimeDeltaFrom
TimestampFrom = DateTimeFrom
GregorianDateTime = DateTime
GregorianDate = Date
JulianDate = JulianDateTime


### For backward compatibility (these are depreciated):

def gmticks(datetime):

    """gmticks(datetime)

       [DEPRECIATED: use the .gmticks() method]
    
       Returns a ticks value based on the values stored in
       datetime under the assumption that they are given in UTC,
       rather than local time.

    """
    return datetime.gmticks()

# Alias
utcticks = gmticks

def tz_offset(datetime,
              # Locals:
              oneSecond=oneSecond):

    """tz_offset(datetime)

       [DEPRECIATED: use the .gmtoffset() method]
    
       Returns a DateTimeDelta instance representing the UTC
       offset for datetime assuming that the stored values refer
       to local time. If you subtract this value from datetime,
       you'll get UTC time.

    """
    return datetime.gmtoffset()

### Constants (only English; see Locale.py for other languages)

# Weekdays
Monday =        0
Tuesday =       1
Wednesday =     2
Thursday =      3
Friday =        4
Saturday =      5
Sunday =        6
# as mapping
Weekday = {'Saturday': 5, 6: 'Sunday', 'Sunday': 6, 'Thursday': 3,
           'Wednesday': 2, 'Friday': 4, 'Tuesday': 1, 'Monday': 0,
           5: 'Saturday', 4: 'Friday', 3: 'Thursday', 2: 'Wednesday',
           1: 'Tuesday', 0: 'Monday'}

# Months
January =       1
February =      2
March =         3
April =         4
May =           5
June =          6
July =          7
August =        8 
September =     9
October =       10
November =      11
December =      12
# as mapping
Month = {2: 'February', 3: 'March', None: 0, 'July': 7, 11: 'November',
    'December': 12, 'June': 6, 'January': 1, 'September': 9, 'August':
    8, 'March': 3, 'November': 11, 'April': 4, 12: 'December', 'May':
    5, 10: 'October', 9: 'September', 8: 'August', 7: 'July', 6:
    'June', 5: 'May', 4: 'April', 'October': 10, 'February': 2, 1:
    'January', 0: None}

# Limits (see also the range checks in mxDateTime.c)
MaxDateTime = DateTime(_sys.maxint / 366 - 1, 12, 31) 
MinDateTime = DateTime(-_sys.maxint / 366 + 2, 1, 1)
MaxDateTimeDelta = DateTimeDeltaFromSeconds(_sys.maxint * 86400.0)
MinDateTimeDelta = -MaxDateTimeDelta

###

class RelativeDateTime:

    """RelativeDateTime(years=0,months=0,days=0,
                  hours=0,minutes=0,seconds=0,
                  year=0,month=0,day=0,
                  hour=None,minute=None,second=None,
                  weekday=None,weeks=None)

       Returns a RelativeDateTime instance for the specified relative
       time. The constructor handles keywords, so you'll only have to
       give those parameters which should be changed when you add the
       relative to an absolute DateTime instance.

       Adding RelativeDateTime instances is supported with the
       following rules: deltas will be added together, right side
       absolute values override left side ones.

       Adding RelativeDateTime instances to DateTime instances will
       return DateTime instances with the appropriate calculations
       applied, e.g. to get a DateTime instance for the first of next
       month, you'd call now() + RelativeDateTime(months=+1,day=1).

    """
    years = 0
    months = 0
    days = 0
    year = None
    month = 0
    day = 0
    hours = 0
    minutes = 0
    seconds = 0
    hour = None
    minute = None
    second = None
    weekday = None

    # cached hash value
    _hash = None

    # For Zope security:
    __roles__ = None
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self,
                 years=0,months=0,days=0,
                 hours=0,minutes=0,seconds=0,
                 year=None,month=None,day=None,
                 hour=None,minute=None,second=None,
                 weekday=None,weeks=0):
        
        self.years = years
        self.months = months
        self.days = days + weeks*7
        self.year = year
        self.month = month
        self.day = day
        self.hours = hours
        self.minutes = minutes
        self.seconds = seconds
        self.hour = hour
        self.minute = minute
        self.second = second
        if weekday is not None:
            #  Make sure we've got a 2-tuple
            assert len(weekday) == 2
            self.weekday = weekday

    def __add__(self,other,
                # Locals:
                isinstance=isinstance):

        if isinstance(other,RelativeDateTime):
            # RelativeDateTime (self) + RelativeDateTime (other)

            r = RelativeDateTime()
            # date deltas
            r.years = self.years + other.years
            r.months = self.months + other.months
            r.days = self.days + other.days
            # absolute entries of other override those in self, if given
            r.year = other.year or self.year
            r.month = other.month or self.month
            r.day = other.day or self.day
            r.weekday = other.weekday or self.weekday
            # time deltas
            r.hours = self.hours + other.hours
            r.minutes = self.minutes + other.minutes
            r.seconds = self.seconds + other.seconds
            # absolute entries of other override those in self, if given
            r.hour = other.hour or self.hour
            r.minute = other.minute or self.minute
            r.second = other.second or self.second
            return r

        else:
            raise TypeError,"can't add the two types"

    def __radd__(self,other,
                 # Locals:
                 isinstance=isinstance,DateTimeType=DateTimeType,
                 DateTime=DateTime,DateTimeDelta=DateTimeDelta):

        if isinstance(other,DateTimeType):
            # DateTime (other) + RelativeDateTime (self)

            # date
            if self.year is None:
                year = other.year + self.years
            else:
                year = self.year + self.years
            if self.month is None:
                month = other.month + self.months
            else:
                month = self.month + self.months
            if self.day is None:
                day = other.day
            else:
                day = self.day
            if day < 0:
                # fix negative day values
                month = month + 1
                day = day + 1
            day = day + self.days
            # time
            if self.hour is None:
                hour = other.hour + self.hours
            else:
                hour = self.hour + self.hours
            if self.minute is None:
                minute = other.minute + self.minutes
            else:
                minute = self.minute + self.minutes
            if self.second is None:
                second = other.second + self.seconds
            else:
                second = self.second + self.seconds

            # Refit into proper ranges:
            if month < 1 or month > 12:
                month = month - 1
                yeardelta, monthdelta = divmod(month, 12)
                year = year + yeardelta
                month = monthdelta + 1

            # Make sure we have integers
            year = int(year)
            month = int(month)
            day = int(day)

            if self.weekday is None:
                return DateTime(year, month, 1) + \
                       DateTimeDelta(day-1,hour,minute,second)
            
            # Adjust to the correct weekday
            day_of_week,index = self.weekday
            d = DateTime(year, month, 1) + \
                DateTimeDelta(day-1,hour,minute,second)
            if index == 0:
                # 0 index: next weekday if no match
                return d + (day_of_week - d.day_of_week)
            elif index > 0:
                # positive index (1 == first weekday of month)
                first = d - (d.day - 1)
                diff = day_of_week - first.day_of_week
                if diff >= 0:
                    return first + (diff + (index-1) * 7)
                else:
                    return first + (diff + index * 7)
            else:
                # negative index (-1 == last weekday of month)
                last = d + (d.days_in_month - d.day)
                diff = day_of_week - last.day_of_week
                if diff <= 0:
                    return last + (diff + (index+1) * 7)
                else:
                    return last + (diff + index * 7)
            
        else:
            raise TypeError,"can't add the two types"

    def __sub__(self,other):

        if isinstance(other,RelativeDateTime):
            # RelativeDateTime (self) - RelativeDateTime (other)

            r = RelativeDateTime()
            # date deltas
            r.years = self.years - other.years
            r.months = self.months - other.months
            r.days = self.days - other.days
            # absolute entries of other override those in self, if given
            r.year = other.year or self.year
            r.month = other.month or self.month
            r.day = other.day or self.day
            r.weekday = other.weekday or self.weekday
            # time deltas
            r.hours = self.hours - other.hours
            r.minutes = self.minutes - other.minutes
            r.seconds = self.seconds - other.seconds
            # absolute entries of other override those in self, if given
            r.hour = other.hour or self.hour
            r.minute = other.minute or self.minute
            r.second = other.second or self.second

            return r

        else:
            raise TypeError,"can't subtract the two types"

    def __rsub__(self,other,
                 # Locals:
                 isinstance=isinstance,DateTimeType=DateTimeType):

        if isinstance(other,DateTimeType):
            # DateTime (other) - RelativeDateTime (self)
            return other + self.__neg__()

        else:
            raise TypeError,"can't subtract the two types"

    def __neg__(self):

        # - RelativeDateTime(self)

        r = RelativeDateTime()
        # negate date deltas
        r.years = - self.years
        r.months = - self.months
        r.days = - self.days
        # absolute entries don't change
        r.year = self.year
        r.month = self.month
        r.day = self.day
        r.weekday = self.weekday
        # negate time deltas
        r.hours = - self.hours
        r.minutes = - self.minutes
        r.seconds = - self.seconds
        # absolute entries don't change
        r.hour = self.hour
        r.minute = self.minute
        r.second = self.second

        return r

    def __nonzero__(self):

        # RelativeDateTime instances are considered false in case
        # they do not define any alterations
        if (self.year is None and
            self.years == 0 and
            self.month is None and
            self.months == 0 and
            self.day is None and
            self.weekday is None and
            self.days == 0 and
            self.hour is None and
            self.hours == 0 and
            self.minute is None and
            self.minutes == 0 and
            self.second is None and
            self.seconds == 0):
            return 0
        else:
            return 1

    def __mul__(self,other):

        # RelativeDateTime (self) * Number (other)
        factor = float(other)

        r = RelativeDateTime()
        # date deltas
        r.years = factor * self.years
        r.months = factor * self.months
        r.days = factor * self.days
        # time deltas
        r.hours = factor * self.hours
        r.minutes = factor * self.minutes
        r.seconds = factor * self.seconds
        return r

    __rmul__ = __mul__

    def __div__(self,other):

        # RelativeDateTime (self) / Number (other)
        return self.__mul__(1/float(other))

    def __eq__(self, other):

        if isinstance(self, RelativeDateTime) and \
           isinstance(other, RelativeDateTime):
            # RelativeDateTime (self) == RelativeDateTime (other)
            if (self.years == other.years and
                self.months == other.months and
                self.days == other.days and
                self.year == other.year and
                self.day == other.day and
                self.hours == other.hours and
                self.minutes == other.minutes and
                self.seconds == other.seconds and
                self.hour == other.hour and
                self.minute == other.minute and
                self.second == other.second and
                self.weekday == other.weekday):
                return 1
            else:
                return 0
        else:
            raise TypeError,"can't compare the two types"

    def __hash__(self):

        if self._hash is not None:
            return self._hash
        x = 1234
        for value in (self.years, self.months, self.days,
                      self.year, self.day,
                      self.hours, self.minutes, self.seconds,
                      self.hour, self.minute, self.second,
                      self.weekday):
            if value is None:
                x = 135051820 ^ x
            else:
                x = hash(value) ^ x
        self._hash = x
        return x

    def __str__(self):

        l = []
        append = l.append

        # Format date part
        if self.year is not None:
            append('%04i-' % self.year)
        elif self.years:
            append('(%0+5i)-' % self.years)
        else:
            append('YYYY-')
        if self.month is not None:
            append('%02i-' % self.month)
        elif self.months:
            append('(%0+3i)-' % self.months)
        else:
            append('MM-')
        if self.day is not None:
            append('%02i' % self.day)
        elif self.days:
            append('(%0+3i)' % self.days)
        else:
            append('DD')
        if self.weekday:
            append(' %s:%i' % (Weekday[self.weekday[0]][:3],self.weekday[1]))
        append(' ')
        
        # Normalize relative time values to avoid fractions
        hours = self.hours
        minutes = self.minutes
        seconds = self.seconds
        hours_fraction = hours - int(hours)
        minutes = minutes + hours_fraction * 60.0
        minutes_fraction = minutes - int(minutes)
        seconds = seconds + minutes_fraction * 6.0
        seconds_fraction = seconds - int(seconds)

        if 0:
            # Normalize to standard time ranges
            if seconds > 60.0:
                extra_minutes, seconds = divmod(seconds, 60.0)
                minutes = minutes + extra_minutes
            elif seconds < -60.0:
                extra_minutes, seconds = divmod(seconds, -60.0)
                minutes = minutes - extra_minutes
            if minutes >= 60.0:
                extra_hours, minutes = divmod(minutes, 60.0)
                hours = hours + extra_hours
            elif minutes <= -60.0:
                extra_hours, minutes = divmod(minutes, -60.0)
                hours = hours - extra_hours

        # Format time part
        if self.hour is not None:
            append('%02i:' % self.hour)
        elif hours:
            append('(%0+3i):' % hours)
        else:
            append('HH:')
        if self.minute is not None:
            append('%02i:' % self.minute)
        elif minutes:
            append('(%0+3i):' % minutes)
        else:
            append('MM:')
        if self.second is not None:
            append('%02i' % self.second)
        elif seconds:
            append('(%0+3i)' % seconds)
        else:
            append('SS')
            
        return ''.join(l)

    def __repr__(self):

        return "<%s instance for '%s' at 0x%x>" % ( 
            self.__class__.__name__, 
            self.__str__(), 
            id(self))

# Alias
RelativeDate = RelativeDateTime

def RelativeDateTimeFrom(*args, **kws):

    """ RelativeDateTimeFrom(*args, **kws)

        Generic RelativeDateTime instance constructor. Can handle
        parsing strings and keywords.

    """
    if len(args) == 1:
        # Single argument
        arg = args[0]
        if _isstring(arg):
            import Parser
            return Parser.RelativeDateTimeFromString(*args, **kws)
        elif isinstance(arg, RelativeDateTime):
            return arg
        else:
            raise TypeError,\
                  'cannot convert argument to RelativeDateTime'

    else:
        return RelativeDateTime(*args, **kws)

def RelativeDateTimeDiff(date1,date2,

                         floor=_math.floor,int=int,divmod=divmod,
                         RelativeDateTime=RelativeDateTime):

    """ RelativeDateTimeDiff(date1,date2)

        Returns a RelativeDateTime instance representing the difference
        between date1 and date2 in relative terms.

        The following should hold: 
        
        date2 + RelativeDateDiff(date1,date2) == date1 

        for all dates date1 and date2.

        Note that due to the algorithm used by this function, not the
        whole range of DateTime instances is supported; there could
        also be a loss of precision.

        XXX There are still some problems left (thanks to Carel
        Fellinger for pointing these out):

        29 1 1901 ->  1 3 1901 = 1 month
        29 1 1901 ->  1 3 1900 = -10 month and -28 days, but
        29 1 1901 -> 28 2 1900 = -11 month and -1 day

        and even worse:

        >>> print RelativeDateDiff(Date(1900,3,1),Date(1901,2,1))
        YYYY-(-11)-DD HH:MM:SS

        with:

        >>> print Date(1901,1,29) + RelativeDateTime(months=-11)
        1900-03-01 00:00:00.00
        >>> print Date(1901,2,1) + RelativeDateTime(months=-11)
        1900-03-01 00:00:00.00

    """
    diff = date1 - date2
    if diff.days == 0:
        return RelativeDateTime()
    date1months = date1.year * 12 + (date1.month - 1)
    date2months = date2.year * 12 + (date2.month - 1)
    #print 'months',date1months,date2months

    # Calculate the months difference
    diffmonths = date1months - date2months
    #print 'diffmonths',diffmonths
    if diff.days > 0:
        years,months = divmod(diffmonths,12)
    else:
        years,months = divmod(diffmonths,-12)
        years = -years
    date3 = date2 + RelativeDateTime(years=years,months=months)
    diff3 = date1 - date3
    days = date1.absdays - date3.absdays
    #print 'date3',date3,'diff3',diff3,'days',days

    # Correction to ensure that all relative parts have the same sign
    while days * diff.days < 0:
        if diff.days > 0:
            diffmonths = diffmonths - 1
            years,months = divmod(diffmonths,12)
        else:
            diffmonths = diffmonths + 1
            years,months = divmod(diffmonths,-12)
            years = -years
        #print 'diffmonths',diffmonths
        date3 = date2 + RelativeDateTime(years=years,months=months)
        diff3 = date1 - date3
        days = date1.absdays - date3.absdays
        #print 'date3',date3,'diff3',diff3,'days',days

    # Drop the fraction part of days
    if days > 0:
        days = int(floor(days))
    else:
        days = int(-floor(-days))

    return RelativeDateTime(years=years,
                            months=months,
                            days=days,
                            hours=diff3.hour,
                            minutes=diff3.minute,
                            seconds=diff3.second)

# Aliases
RelativeDateDiff = RelativeDateTimeDiff
Age = RelativeDateTimeDiff

###

_current_year = now().year
_current_century, _current_year_in_century = divmod(_current_year, 100)
_current_century = _current_century * 100

def add_century(year,

                current_year=_current_year,
                current_century=_current_century):
    
    """ Sliding window approach to the Y2K problem: adds a suitable
        century to the given year and returns it as integer.

        The window used depends on the current year (at import time).
        If adding the current century to the given year gives a year
        within the range current_year-70...current_year+30 [both
        inclusive], then the current century is added. Otherwise the
        century (current + 1 or - 1) producing the smallest difference is
        chosen.

    """
    if year > 99:
        # Take it as-is
        return year
    year = year + current_century
    diff = year - current_year
    if diff >= -70 and diff <= 30:
        return year
    elif diff < -70:
        return year + 100
    else:
        return year - 100

# Reference formulas for JDN taken from the Calendar FAQ:

def gregorian_jdn(year,month,day):

    # XXX These require proper integer division.
    a = (14-month)/12
    y = year+4800-a
    m = month + 12*a - 3
    return day + (306*m+5)/10 + y*365 + y/4 - y/100 + y/400 - 32045

def julian_jdn(year,month,day):

    # XXX These require proper integer division.
    a = (14-month)/12
    y = year+4800-a
    m = month + 12*a - 3
    return day + (306*m+5)/10 + y*365 + y/4 - 32083



from mx import Proxy
import sys

cleaned_up = 0

class DataRecord:
    a = 2
    b = 3
    # Make read-only:
    def __public_setattr__(self,what,to):
        raise Proxy.AccessError,'read-only'
    # Cleanup protocol
    def __cleanup__(self):
        global cleaned_up
        print 'cleaning up',self
        cleaned_up = 1

o = DataRecord()

# Wrap the instance:
p = Proxy.InstanceProxy(o,('a',))
# Remove o from the accessible system:
del o

print 'Read p.a through Proxy:',p.a

# This will cause an exception, because the object is read-only
if 1:
    try:
        p.a = 3
    except:
        pass
    else:
        raise AssertionError,'should be read-only'

# This will cause an exception, because no access is given to .b
if 1:
    try:
        p.b
    except Proxy.AccessError:
        pass
    else:
        raise AssertionError,'should give no access'

# Cleanup error tracebacks by overwriting the previous error;
# this should remove the reference to p in the traceback of the
# previous exception.
if 1:
    try:
        1/0
    except:
        pass

# Deleting the Proxy will also delete the wrapped object, if there
# is no other reference to it in the system. It will invoke
# the __cleanup__ method in that case.
del p
assert cleaned_up

#
# Creating and deleting weak Proxies and their referenced objects
#
d = {}
p = Proxy.WeakProxy(d)
q = Proxy.WeakProxy(d)
del p
del q
del d

d = {}
p = Proxy.WeakProxy(d)
q = Proxy.WeakProxy(d)
del q
del p
del d

d = {}
p = Proxy.WeakProxy(d)
q = Proxy.WeakProxy(d)
del d
del p
del q

d = {}
p = Proxy.WeakProxy(d)
q = Proxy.WeakProxy(d)
del d
del q
del p

d = {}
p = Proxy.WeakProxy(d)
q = Proxy.WeakProxy(d)
del d
Proxy.checkweakrefs()
try:
    p[1]
except Proxy.LostReferenceError:
    pass
else:
    print '*** Phantom object still alive !'

print
print 'Works.'



#!/usr/local/bin/python

from mx.BeeBase import BeeStorage, BeeDict

BeeStorage._debug = 1
BeeDict._debug = 1

def part1():

    # Fill initial version of the dictionary
    d = BeeDict.BeeDict('testernesto', autocommit=1)
    for i in range(1000):
        d[i] = "A"*i
    d.commit()
    #d.free_cache()
    d.close()


def part2():

    # Update a few values
    d = BeeDict.BeeDict('testernesto', autocommit=1)
    for i in range(100,300):
        d[i] = "C" * i
    d.commit()

import sys
if sys.argv[1] == 'init':
    import os
    for filename in ('testernesto.dat',
                     'testernesto.idx',
                     ):
      try:
        os.remove(filename)
      except:
        pass
    part1()

else:
    part2()
                



""" A pure Python Queue implementation modelled after mxQueue.

    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""

__version__ = '0.1'

class UserQueue:

    def __init__(self):

        self.queue = []
        self.headindex = 0

    def push(self,x):

        self.queue.append(x)

    def pop(self):

        headindex = self.headindex
        queue = self.queue
        x = queue[headindex]
        queue[headindex] = None # del. reference
        headindex = headindex + 1
        if headindex > 100  and headindex > len(queue) / 2:
            # compactify
            self.queue = self.queue[headindex:]
            self.headindex = 0
        else:
            self.headindex = headindex
        return x

    def not_empty(self):

        return self.headindex != len(self.queue)

    def head(self):

        return self.queue[self.headindex]

    def tail(self):

        return self.queue[-1]

    def __len__(self):

        return len(self.queue) - self.headindex

    def __repr__(self):

        l = self.queue[self.headindex:]
        l.reverse()
        return '<UserQueue [%s]>' % repr(l)[1:-1]

    def __str__(self):

        l = self.queue[self.headindex:]
        l.reverse()
        return 'q' + repr(l)



""" BeeBase - A suite of building parts for databases.

    Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.

    HTML-Logo:

    <FONT COLOR="#FF3333"><SUP><B>Bee</B></SUP></FONT><B>Base</B></FONT>

"""#"

# Import version from C extension
from mxBeeBase import __version__




# -*- coding: latin-1 -*-

""" Timezone information.

    XXX This module still has prototype status and is undocumented.

    XXX Double check the offsets given in the zonetable below.

    XXX Add TZ environment variable parsing functions. The REs are already
        there.

    Copyright (c) 1998-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.

"""
import DateTime
import re

### REs

# time zone parsing
isozone = ('(?P<zone>[+-]\d\d:?(?:\d\d)?|Z)')
zone = ('(?P<zone>[A-Z]+|[+-]\d\d?:?(?:\d\d)?)')
zoneoffset = ('(?:'
              '(?P<zonesign>[+-])?'
              '(?P<hours>\d\d?)'
              ':?'
              '(?P<minutes>\d\d)?'
              '(?P<extra>\d+)?'
              ')'
              )

# TZ environment variable parsing
dstswitchtime = ('(?P<hour>\d\d?):?'
                 '(?P<minute>\d\d)?:?'
                 '(?P<second>\d\d)?')
dstswitch = ('(?:'
              '(?P<doy>\d+)|'
              '(?:J(?P<jdoy>\d+))|'
              '(?:M(?P<month>\d+).(?P<week>\d+).(?P<day>\d+))'
             ')'
             '(?:/' + dstswitchtime + ')?'
             )

# XXX Doesn't work since re doesn't like multiple occurrences of
#     group names.
#tz = ('(?::(?P<filename>.+))|'
#      '(?P<std>[A-Z]+)' + zoneoffset + 
#      '(?:'
#       '(?P<dst>[A-Z]+)' + zoneoffset + '?'+
#       '(?:[;,]' + dstswitch + '[;,]' + dstswitch + ')'
#      ')?'
#      )

# Compiled RE objects
isozoneRE = re.compile(zone)
zoneRE = re.compile(zone)
zoneoffsetRE = re.compile(zoneoffset)
#tzRE= re.compile(tz)

### Time zone offset table
#
# The offset given here represent the difference between UTC and the
# given time zone.
#
# Additions and corrections are always welcome :-)
#
# Note that some zone names are ambiguous, e.g. IST can refer to Irish
# Summer Time, Indian Standard Time, Israel Standard Time. We've
# usualy chosen meaning with the most wide-spread use.
#
zonetable = {
    # Timezone abbreviations
    # Std     Summer

    # Standards
    'UT':0,
    'UTC':0,
    'GMT':0,

    # A few common timezone abbreviations
    'CET':1,  'CEST':2, 'CETDST':2, # Central European
    'MET':1,  'MEST':2, 'METDST':2, # Mean European
    'MEZ':1,  'MESZ':2,             # Mitteleuropäische Zeit
    'EET':2,  'EEST':3, 'EETDST':3, # Eastern Europe
    'WET':0,  'WEST':1, 'WETDST':1, # Western Europe
    'MSK':3,  'MSD':4,  # Moscow
    'IST':5.5,          # India
    'JST':9,            # Japan
    'KST':9,            # Korea
    'HKT':8,            # Hong Kong
    'BST':1,            # British Summer Time

    # US time zones
    'AST':-4, 'ADT':-3, # Atlantic
    'EST':-5, 'EDT':-4, # Eastern
    'CST':-6, 'CDT':-5, # Central
    'MST':-7, 'MDT':-6, # Midwestern
    'PST':-8, 'PDT':-7, # Pacific

    # Australian time zones
    'CAST':9.5, 'CADT':10.5, # Central
    'EAST':10,  'EADT':11,   # Eastern
    'WAST':8,   'WADT':9,    # Western
    'SAST':9.5, 'SADT':10.5, # Southern

    # US military time zones
    'Z': 0,
    'A': 1,
    'B': 2,
    'C': 3,
    'D': 4,
    'E': 5,
    'F': 6,
    'G': 7,
    'H': 8,
    'I': 9,
    'K': 10,
    'L': 11,
    'M': 12,
    'N':-1,
    'O':-2,
    'P':-3,
    'Q':-4,
    'R':-5,
    'S':-6,
    'T':-7,
    'U':-8,
    'V':-9,
    'W':-10,
    'X':-11,
    'Y':-12
    }    

def utc_offset(zone,

               zoneoffset=zoneoffsetRE,
               zonetable=zonetable,zerooffset=DateTime.DateTimeDelta(0),
               oneMinute=DateTime.oneMinute):

    """ utc_offset(zonestring)

        Return the UTC time zone offset as DateTimeDelta instance.

        zone must be string and can either be given as +-HH:MM,
        +-HHMM, +-HH numeric offset or as time zone
        abbreviation. Daylight saving time must be encoded into the
        zone offset.

        Timezone abbreviations are treated case-insensitive.

    """
    if not zone:
        return zerooffset
    uzone = zone.upper()
    if zonetable.has_key(uzone):
        return zonetable[uzone]*DateTime.oneHour
    offset = zoneoffset.match(zone)
    if not offset:
        raise ValueError,'wrong format or unknown time zone: "%s"' % zone
    zonesign,hours,minutes,extra = offset.groups()
    if extra:
        raise ValueError,'illegal time zone offset: "%s"' % zone
    offset = int(hours or 0) * 60 + int(minutes or 0)
    if zonesign == '-':
        offset = -offset
    return offset*oneMinute




#!/usr/local/bin/python -O
""" queuebench - queue implementation benchmark

    Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
    See the documentation for further information on copyrights,
    or contact the author. All Rights Reserved.
"""
import time
from mx import Queue
import UserQueue
from sys import argv, exit

try:
    numtests, pushes, pops = eval(argv[1]), eval(argv[2]), eval(argv[3])
    assert pushes >= pops
except:
    print 'usage: queuebench.py <ntests> <pushes> <pops>, where <pushes> >= <pops>'
    exit(1)
 
def test(reps, func):
    start_cpu  = time.clock()
    for i in xrange(reps):
        x = func()
    return time.clock() - start_cpu
 
def method1():
    x = []                                           # built-in list
    push = x.append
    for i in range(pushes): push('spam'+'i')
    for i in range(pops):   top = x[0]; del x[0]

def method3():
    s = Queue.Queue()                              # Queue
    push = s.push
    pop = s.pop
    for i in range(pushes): push('spam'+'i')
    for i in range(pops):   top = pop()

if 0:
    def method3a():
        s = Queue.Queue()                              # Queue
        push = s.push
        for i in range(pushes): push('spam'+'i')
        t = s.pop_many(pops)                             # pop all at once

def method3b():
    s = Queue.Queue()                              # Queue
    push = s.push
    for i in range(pushes): s << ('spam'+'i')
    for i in range(pops):   top = s >> 1

if 0:
    def method3c():
        s = Queue.Queue()                              # Queue
        l = [''] * pushes
        for i in range(pushes): l[i] = ('spam'+'i')
        s.push_many(l)
        s.pop_many(pops)

def method4():
    s = UserQueue.UserQueue()                         # UserQueue
    push = s.push
    pop = s.pop
    for i in range(pushes): push('spam'+'i')
    for i in range(pops):   top = pop()

print 'list: ', test(numtests, method1)            # run func 20 tests
print 'Queue (with push + pop):', test(numtests, method3)
#print 'Queue (with push + pop_many):', test(numtests, method3a)
print 'Queue (with << + >>):', test(numtests, method3b)
#print 'Queue (with push_many + pop_many):', test(numtests, method3c)
print 'UserQueue:', test(numtests, method4)
 




import hack
import mx.Tools.NewBuiltins

k = range(10000)
l = range(1,10001)
loops = trange(100)

def f(k=k,l=l,tuples=tuples,loops=loops):
    for i in loops:
        for a,b in tuples(k,l):
            pass

def f1(k=k,l=l,lists=lists,loops=loops):
    for i in loops:
        for a,b in lists(k,l):
            pass

def g(k=k,l=l,map=map,loops=loops):
    for i in loops:
        for a,b in map(None,k,l):
            pass

def h(k=k,l=l,indices=indices,len=len,loops=loops):
    for i in loops:
        for i in indices(k):
            a,b = k[i], l[i]

print 'with tuples():',
hack.clock('f()')
print 'with lists():',
hack.clock('f1()')
print 'with map():',
hack.clock('g()')
print 'indexed:',
hack.clock('h()')
print 'map(None,...):',
hack.clock('apply(map,(None,)+(k,)*100)')
print 'tuples(...):',
hack.clock('tuples((k,)*100)')
print 'lists(...):',
hack.clock('lists((k,)*100)')

# Check
assert apply(map,(None,)+(k,)*100) == tuples((k,)*100)



from mx.TextTools.Examples.HTML import *

def test():
    text = """<frame noresize>"""

    utext = upper(text)
    print "UPPER:", utext
    result, taglist, nextindex = tag(utext,htmltable)
    print_tags(text,taglist)

    print
    utext = lower(text)
    print "LOWER", utext
    result, taglist, nextindex = tag(utext,htmltable)
    print_tags(text,taglist)

test()
